@@ -62,6 +62,10 @@
 #define LOG
 */
 
+#ifdef LOG
+#include <stdarg.h>
+#endif
+
 /* FIXME: browser exit on URL box destroy, segfault on 'close' click
 #define URLBOX
 */
@@ -111,8 +115,9 @@ typedef struct {
 } plugin_instance_t;
 
 
-static void xprintf (const char *format, ...) {
-
+static void __attribute__ ((format (printf, 1, 2)))
+xprintf (const char *format, ...)
+{
 #ifdef LOG
   static FILE *log_fd=NULL;
 
@@ -163,8 +168,8 @@ char *NPP_GetMIMEDescription(void) {
 	 ;
 }
 
-NPError NPP_GetValue(void *future, NPPVariable variable, void *value){
-
+NPError NPP_GetValue (NPP instance, NPPVariable variable, void *value)
+{
   NPError err = NPERR_NO_ERROR;
   xprintf("NPP_GetValue: variable=%d\n", variable);
 
@@ -290,10 +295,10 @@ static void launch_gxine (plugin_instance_t *this) {
   }
 
   if (!fork2()) {
-    char executable[4096];
+    char executable[FILENAME_MAX];
     char arg[4096];
 
-    snprintf(executable, 4096, "%s/gxine", GXINE_BINDIR);
+    snprintf(executable, sizeof (executable), "%s/gxine", GXINE_BINDIR);
 
     if (this->emu_mode == EMU_MPLAYER)
       snprintf(arg, sizeof (arg), "mms%s", globals.url);
@@ -496,8 +501,8 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window) {
   this->top_level = XtWindowToWidget (this->display, this->window);
   this->screen    = XtScreen (this->top_level);
 
-  xprintf("x=%lu, y=%lu, w=%lu, h=%lu\n", window->x, window->y, window->width, window->height);
-  xprintf("window = %lu NPERR_NO_ERROR\n", this->window);
+  xprintf("x=%u, y=%u, w=%u, h=%u\n", window->x, window->y, window->width, window->height);
+  xprintf("window = %u NPERR_NO_ERROR\n", this->window);
 
   this->black = BlackPixelOfScreen (this->screen);
   this->white = WhitePixelOfScreen (this->screen);
