@@ -1,8 +1,8 @@
 /*
  * Copyright (C) 2002-2005 the xine-project
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
@@ -31,7 +31,9 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <dlfcn.h>
+#ifdef HAVE_ALLOCA_H
 #include <alloca.h>
+#endif
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -46,23 +48,17 @@ typedef enum { GNOME_VFS_OK } GnomeVFSResult; /* incomplete */
 /*
  * the bunch of gnome vfs functions needed
  */
- 
-struct gnome_vfs_functions 
+
+struct gnome_vfs_functions
 {
-  void  (* application_registry_remove_application) ( const char *app_id );
-  void  (* application_registry_add_mime_type)      ( const char *app_id, const char *mime_type );
-  void  (* application_registry_set_value)          ( const char *app_id, const char *key, const char *value);
-  void  (* application_registry_set_bool_value)     ( const char *app_id, const char *key, int value);
-  void  (* application_registry_remove_mime_type)   ( const char *app_id, const char *mime_type );
-
+  void (*application_registry_remove_application) (const char *app_id);
+  void (*application_registry_add_mime_type)	  (const char *app_id, const char *mime_type);
+  void (*application_registry_set_value)	  (const char *app_id, const char *key, const char *value);
+  void (*application_registry_set_bool_value)	  (const char *app_id, const char *key, int value);
+  void (*application_registry_remove_mime_type)	  (const char *app_id, const char *mime_type);
 #ifndef GNOME_VFS_DISABLE_DEPRECATED
-  void  (* mime_freeze)                      ( void );
-  void  (* mime_thaw)                        ( void );
-  void  (* mime_registered_mime_type_delete) ( const char * mime_type );
-
-  GnomeVFSResult (* mime_set_default_application)           ( const char *mime_type, const char *application_id );
-  GnomeVFSResult (* mime_set_registered_type_key)           ( const char *mime_type, const char *key, const char *data );
-  GnomeVFSResult (* mime_set_value)                         ( const char *mime_type, const char *key, const char *value );
+  void (*mime_freeze) (void);
+  void (*mime_thaw)   (void);
 #endif
 };
 
@@ -77,11 +73,11 @@ struct mime_info
   const char * extensions;
   const char * icon_filename;
 };
-    
+
 /*
  * make the code read as normal gnome app
  */
- 
+
 #define gnome_vfs_application_registry_remove_application gnome_vfs->application_registry_remove_application
 #define gnome_vfs_application_registry_add_mime_type gnome_vfs->application_registry_add_mime_type
 #define gnome_vfs_application_registry_remove_mime_type gnome_vfs->application_registry_remove_mime_type
@@ -92,7 +88,7 @@ struct mime_info
 #define gnome_vfs_mime_freeze gnome_vfs->mime_freeze
 
 static void * gnome_vfs_handle = NULL;
-
+static int gnome_vfs_available = -1;
 static struct gnome_vfs_functions * gnome_vfs = NULL;
 
 
@@ -113,34 +109,37 @@ static func_t get_func (const char *name)
 
 #define GET_FUNC(VAR,NAME) \
   if (!(VAR = (typeof (VAR)) get_func ((NAME)))) \
-    return 0
+    return gnome_vfs_available = 0
 
 #define GET_FUNC_S(VAR) \
   if (!(gnome_vfs->VAR = (typeof (gnome_vfs->VAR)) \
 			 get_func ("gnome_vfs_" #VAR))) \
-    return 0
+    return gnome_vfs_available = 0
 
-static int gnome_vfs_init (void)
+gboolean di_gnome_vfs_init (void)
 {
   void (*gnome_vfs_init_func) (void) = NULL;
   void (*gnome_vfs_mime_get_value) (const char *, const char *) = NULL;
-    
+
+  if (gnome_vfs_available != -1)
+    return gnome_vfs_available;
+
   /* load the gnomevfs library */
   gnome_vfs_handle = dlopen ("libgnomevfs-2.so.0", RTLD_LAZY);
   if (!gnome_vfs_handle)
   {
     fprintf (stderr, "%s.\n", dlerror());
-    return 0;
+    return gnome_vfs_available = 0;
   }
-    
-  gnome_vfs = ( struct gnome_vfs_functions * ) calloc( 1, sizeof( struct gnome_vfs_functions ) );     
-  
+
+  gnome_vfs = (struct gnome_vfs_functions *) calloc (1, sizeof (struct gnome_vfs_functions));
+
   /* this call causes gnome-vfs to become initialized */
   GET_FUNC (gnome_vfs_init_func, "gnome_vfs_init");
   gnome_vfs_init_func ();
 
   GET_FUNC (gnome_vfs_mime_get_value, "gnome_vfs_mime_get_value");
-  gnome_vfs_mime_get_value( "text/plain", "description" );
+  gnome_vfs_mime_get_value ("text/plain", "description");
 
   /* extract needed symbols from the library */
   GET_FUNC_S (mime_freeze);
@@ -151,14 +150,14 @@ static int gnome_vfs_init (void)
   GET_FUNC_S (application_registry_set_bool_value);
   GET_FUNC_S (application_registry_remove_mime_type);
 
-  return 1;
+  return gnome_vfs_available = 1;
 }
 
 /*
  * release the gnome-vfs library and free the function pointer struct
  */
 
-#undef GET_FUNC 
+#undef GET_FUNC
 #define GET_FUNC(VAR,NAME) \
   if ((VAR = (typeof (VAR)) get_func ((NAME))) == NULL) \
     return
@@ -169,14 +168,14 @@ static void gnome_vfs_shutdown (void)
   if (gnome_vfs_handle)
   {
     void (*gnome_vfs_application_registry_sync) (void) = NULL;
-    
+
     /* extract and call shutdown function */
     GET_FUNC (gnome_vfs_application_registry_sync, "gnome_vfs_application_registry_sync");
 
     gnome_vfs_application_registry_sync ();
     dlclose (gnome_vfs_handle);
   }
-  
+
   free (gnome_vfs);
 }
 
@@ -202,7 +201,7 @@ di_report (const char *fmt, ...)
   if (section)
   {
     asreprintf (&report, _("%s%s<i>Errors while registering with %s:</i>\n"),
-		report ? : "", report ? "\n" : "", section);
+		report ? : "", report ? "\n\n" : "", section);
     section = NULL;
   }
 
@@ -245,34 +244,34 @@ static void di_fclose (FILE *f, const char *fname)
 /*
  *  add the installed application to the gnome application info registry
  */
-static GnomeVFSResult register_application( const char * application_id, 
-					    const char * product_name, 
-					    const char * product_version,
-					    const char * command, 
-					    const char * install_path ) {
+static GnomeVFSResult
+register_application (const char *application_id, const char *product_name,
+		      const char *product_version, const char *command,
+		      const char *install_path)
+{
+  char *command_full = (char *) alloca (strlen (install_path) + strlen (command) + 10);
+  char *product = (char *) alloca (strlen (product_name) + strlen (product_version) + 2);
+
+  sprintf (command_full, "%s/%s", install_path, command);
+  sprintf (product, "%s %s", product_name, product_version);
 
-  char * command_full = (char *) alloca( strlen( install_path ) + strlen( command ) + 10 );
-  char * product = (char *) alloca( strlen( product_name ) + strlen( product_version ) + 2 );
-  
-  sprintf( command_full, "%s/%s", install_path, command );
-  sprintf( product, "%s %s", product_name, product_version );
-  
   /* entry will be created automatically */
-  gnome_vfs_application_registry_set_value( application_id, "name", product );
-  gnome_vfs_application_registry_set_value( application_id, "command", command_full );
-  gnome_vfs_application_registry_set_bool_value( application_id, "can_open_multiple_files", 1 );
-  gnome_vfs_application_registry_set_bool_value( application_id, "requires_terminal", 0 );
-  
+  gnome_vfs_application_registry_set_value (application_id, "name", product);
+  gnome_vfs_application_registry_set_value (application_id, "command", command_full);
+  gnome_vfs_application_registry_set_bool_value (application_id, "can_open_multiple_files", 1);
+  gnome_vfs_application_registry_set_bool_value (application_id, "requires_terminal", 0);
+
   return GNOME_VFS_OK;
 }
 
-void di_register_gnome (void) {
-
+void di_register_gnome (void)
+{
   char *mime_types;
 
   section = "GNOME";
 
-  if (!gnome_vfs_init()) {
+  if (!di_gnome_vfs_init())
+  {
     di_report (_("GNOME initialisation failed"));
     return;
   }
@@ -287,22 +286,20 @@ void di_register_gnome (void) {
    */
 
   mime_types = xine_get_mime_types (xine);
-  while (mime_types) {
-
+  while (mime_types)
+  {
     char *next = mime_types;
 
-    if ( (next=strchr (mime_types, ';')) ) {
-      char *end;
-
+    if ((next = strchr (mime_types, ';')))
+    {
       *next=0;
-
-      end = strchr (mime_types, ':');
-      if (end) {
+      char *end = strchr (mime_types, ':');
+      if (end)
+      {
 	*end=0;
-
 	g_print (_("desktop_integration: registering xine for MIME type ‘%s’\n"),
 		mime_types);
-	gnome_vfs_application_registry_add_mime_type ("gxine", mime_types );  
+	gnome_vfs_application_registry_add_mime_type ("gxine", mime_types);
       }
     }
 
@@ -311,27 +308,25 @@ void di_register_gnome (void) {
 
   /* force the user.mime and user.keys to be written  */
   gnome_vfs_mime_thaw();
-  
+
   gnome_vfs_shutdown();
 }
 
-void di_register_kde (void) {
-
+void di_register_kde (void)
+{
   gchar *fname;
   gchar *kdehome;
   FILE  *f;
   char  *mime_types;
-  
+
   section = "KDE";
 
   kdehome = getenv ("KDEHOME");
 
-  if (kdehome) 
-    fname = g_strconcat (kdehome, "/share/applnk/Multimedia", 
-			 NULL);
+  if (kdehome)
+    fname = g_build_filename (kdehome, "share/applnk/Multimedia", NULL);
   else
-    fname = g_strconcat (g_get_home_dir(), "/.kde/share/applnk/Multimedia", 
-			 NULL);
+    fname = g_build_filename (g_get_home_dir(), ".kde/share/applnk/Multimedia", NULL);
   if (ensure_path_exists (fname, 0755))
   {
     di_report (_("can't create directory %s: %s"), fname, strerror (errno));
@@ -344,16 +339,14 @@ void di_register_kde (void) {
    * produce gxine.desktop
    */
 
-  if (kdehome) 
-    fname = g_strconcat (kdehome, "/share/applnk/Multimedia/gxine.desktop", 
-			 NULL);
+  if (kdehome)
+    fname = g_build_filename (kdehome, "share/applnk/Multimedia", "gxine.desktop", NULL);
   else
-    fname = g_strconcat (g_get_home_dir(), "/.kde/share/applnk/Multimedia/gxine.desktop", 
-			 NULL);
-  
-  
+    fname = g_build_filename (g_get_home_dir(), ".kde/share/applnk/Multimedia", "gxine.desktop", NULL);
+
   f = di_fopen (fname);
-  if (!f) {
+  if (!f)
+  {
     g_free (fname);
     return;
   }
@@ -380,37 +373,33 @@ void di_register_kde (void) {
    */
 
   mime_types = xine_get_mime_types (xine);
-  while (mime_types) {
-
+  while (mime_types)
+  {
     char *next = mime_types;
 
-    if ( (next=strchr (mime_types, ';')) ) {
-      char *end;
-
+    if ((next = strchr (mime_types, ';')))
+    {
       *next=0;
-
-      end = strchr (mime_types, ':');
-      if (end) {
+      char *end = strchr (mime_types, ':');
+      if (end)
+      {
 	*end=0;
-
 	g_print (_("desktop_integration: KDE registering xine for MIME type ‘%s’\n"),
 		mime_types);
-
 	fprintf (f, "%s;", mime_types);
-
       }
     }
 
     mime_types = next ? next + 1 : NULL;
   }
-  
+
   fprintf (f, "\n");
   di_fclose (f, fname);
   g_free (fname);
 }
 
-void di_register_mailcap (void) {
-
+void di_register_mailcap (void)
+{
   char  *mc;
   gchar *fname;
   ssize_t n;
@@ -419,12 +408,13 @@ void di_register_mailcap (void) {
 
   section = "mailcap";
 
-  fname = g_strconcat (g_get_home_dir(), "/.mailcap", NULL);
+  fname = g_build_filename (g_get_home_dir(), ".mailcap", NULL);
 
   mc = read_entire_file (fname, &n);
 
   f = di_fopen (fname);
-  if (!f) {
+  if (!f)
+  {
     g_free (fname);
     return;
   }
@@ -438,36 +428,34 @@ void di_register_mailcap (void) {
    */
 
   mime_types = xine_get_mime_types (xine);
-  while (mime_types) {
-
+  while (mime_types)
+  {
     char *next = mime_types;
 
-    if ( (next=strchr (mime_types, ';')) ) {
-      char *end;
-
+    if ((next = strchr (mime_types, ';')))
+    {
       *next=0;
-
-      end = strchr (mime_types, ':');
-      if (end) {
+      char *end = strchr (mime_types, ':');
+      if (end)
+      {
 	*end=0;
-
-	if (!mc || !strstr (mc, mime_types)) {
-
+	if (!mc || !strstr (mc, mime_types))
+	{
 	  g_print (_("desktop_integration: mailcap: registering xine for MIME type ‘%s’\n"),
 		  mime_types);
-	  
+
 	  fprintf (f, "%s;%s/gxine %%s\n", mime_types,
 		   bindir);
-	} else { /* MIME type already registered */
+	}
+	else /* MIME type already registered */
 	  g_print (_("desktop_integration: %s already has a handler\n"),
 		  mime_types);
-	}
       }
     }
 
     mime_types = next ? next + 1 : NULL;
   }
-  
+
   fprintf (f, "\n");
   di_fclose (f, fname);
   g_free (fname);
@@ -480,11 +468,11 @@ void di_register_mozilla (void)
   unsigned int i;
   struct stat st;
 
-  static const char *dir[] = { "/.mozilla", "/.mozilla/firefox", "/.firefox" };
+  static const char *dir[] = { ".mozilla", ".mozilla/firefox", ".firefox" };
 
   section = "Mozilla";
 
-  sname = g_strconcat (plugindir, "/gxineplugin.so", NULL);
+  sname = g_build_filename (plugindir, "gxineplugin.so", NULL);
   if (stat (sname, &st))
     return;
 
@@ -493,7 +481,7 @@ void di_register_mozilla (void)
   for (i = 0; i < G_N_ELEMENTS (dir); ++i)
   {
     /* just make sure the directory exists */
-    fname = g_strconcat (g_get_home_dir(), dir[i], "/plugins", NULL);
+    fname = g_build_filename (g_get_home_dir(), dir[i], "plugins", NULL);
     if (ensure_path_exists (fname, 0755))
     {
       di_report (_("can't create directory %s: %s"), fname, strerror (errno));
@@ -501,7 +489,7 @@ void di_register_mozilla (void)
     }
     g_free (fname);
 
-    fname = g_strconcat (g_get_home_dir(), dir[i], "/plugins/gxineplugin.so", NULL);
+    fname = g_build_filename (g_get_home_dir(), dir[i], "plugins/gxineplugin.so", NULL);
     buf[0] = 0;
 
     /* If the destination object exists, check if it's a symlink and if it's
@@ -509,7 +497,7 @@ void di_register_mozilla (void)
      * the object (if it exists) and create a fresh symlink in its place.
      */
 
-    if (readlink (fname, buf, sizeof (buf)) &&
+    if (readlink (fname, buf, sizeof (buf)) == -1 &&
         errno != EINVAL && errno != ENOENT)
     {
       di_report (_("symlink check on %s failed: %s"), fname, strerror (errno));
