@@ -101,11 +101,6 @@ static void gtk_video_size_request (GtkWidget      *widget,
 static void gtk_video_size_allocate (GtkWidget      *widget,
  				     GtkAllocation  *allocation);
 
-static GList *gtv_create_plugins (gtk_video_private_t *priv,
-				  xine_audio_port_t **audio,
-				  xine_video_port_t **video,
-				  const char *plugins, int type);
-
 static gboolean gtv_unblank_screen (GtkVideo *);
 
 static GtkWidgetClass *parent_class = NULL;
@@ -126,7 +121,7 @@ static pthread_mutex_t resize_add_lock = PTHREAD_MUTEX_INITIALIZER;
   } while (0)
 
 /*
- * private data 
+ * private data
  */
 
 struct gtk_video_private_s {
@@ -178,10 +173,12 @@ struct gtk_video_private_s {
 #ifdef WITH_DBUS
   DBusGConnection	*connection;
   DBusGProxy		*gs_proxy;
+  guint32		 cookie;
 #endif
 
+  gboolean		 playing_no_blank:8;
   /* visualization */
-  gboolean		   vis_active;
+  gboolean		   vis_active:8;
   xine_post_t		  *vis_plugin;
   char			  *vis_plugin_id;
   xine_audio_port_t       *vis_audio;
@@ -191,6 +188,7 @@ struct gtk_video_private_s {
     GList	*list;
     gboolean	 enable;
   }			   post_deinterlace, post_video, post_audio;
+  gboolean		   have_tvtime;
 };
 
 static int gtv_table_signals[LAST_SIGNAL] = { 0 };
@@ -229,6 +227,16 @@ static inline gboolean is_fullscreen (gtk_video_private_t *priv)
   return priv->fullscreen_mode == 1;
 }
 
+static gboolean gtv_test_inhibit_blank (GtkVideo *gtv)
+{
+  /* (fullscreen || (deny blank && drawable && playing)) && !logo && !paused */
+  return (is_fullscreen (gtv->priv)
+	  || (gtv->priv->playing_no_blank && GTK_WIDGET_DRAWABLE (&gtv->widget)
+	      && xine_get_status (gtv->priv->stream) == XINE_STATUS_PLAY))
+	 && !playlist_showing_logo ()
+	 && xine_get_param (gtv->priv->stream, XINE_PARAM_SPEED);
+}
+
 #ifdef WITH_DBUS
 static inline gboolean
 screensaver_is_running_dbus (gtk_video_private_t *priv)
@@ -247,17 +255,43 @@ gtv_screensaver_inhibit_dbus (gtk_video_private_t *priv, gboolean inhibit)
 
   if (inhibit)
   {
+    char *application;
     char *reason;
+    guint32 cookie;
+
+    application = g_strdup ("gxine");
     reason = g_strdup (_("gxine is active"));
-    res = dbus_g_proxy_call (priv->gs_proxy, "InhibitActivation", &error,
+
+    res = dbus_g_proxy_call (priv->gs_proxy, "Inhibit", &error,
+			     G_TYPE_STRING, application,
 			     G_TYPE_STRING, reason,
-			     G_TYPE_INVALID, G_TYPE_INVALID);
+			     G_TYPE_INVALID,
+			     G_TYPE_UINT, &cookie,
+			     G_TYPE_INVALID);
+    if (res)
+      /* save the cookie */
+      priv->cookie = cookie;
+    else
+      /* old API */
+      res = dbus_g_proxy_call (priv->gs_proxy, "InhibitActivation", &error,
+			       G_TYPE_STRING, reason,
+			       G_TYPE_INVALID, G_TYPE_INVALID);
     g_free (reason);
+    g_free (application);
   }
   else
-    res = dbus_g_proxy_call (priv->gs_proxy, "AllowActivation", &error,
-			     G_TYPE_INVALID, G_TYPE_INVALID, G_TYPE_INVALID);
-
+  {
+    res = dbus_g_proxy_call (priv->gs_proxy, "UnInhibit", &error,
+			     G_TYPE_UINT, priv->cookie,
+			     G_TYPE_INVALID, G_TYPE_INVALID);
+    if (res)
+      /* clear the cookie */
+      priv->cookie = 0;
+    else
+      /* old API */
+      res = dbus_g_proxy_call (priv->gs_proxy, "AllowActivation", &error,
+			      G_TYPE_INVALID, G_TYPE_INVALID, G_TYPE_INVALID);
+  }
   if (!res && error)
     g_printerr (
 	     _("gtkvideo: problem when inhibiting the GNOME screensaver: %s\n"),
@@ -266,6 +300,13 @@ gtv_screensaver_inhibit_dbus (gtk_video_private_t *priv, gboolean inhibit)
     g_error_free (error);
 }
 
+static void
+gtv_screensaver_maybe_inhibit_dbus (gtk_video_private_t *priv, gboolean inhibit)
+{
+  if (screensaver_is_running_dbus (priv))
+    gtv_screensaver_inhibit_dbus (priv, inhibit);
+}
+
 static void
 gtv_gs_proxy_destroy_cb (GObject *obj, gtk_video_private_t *priv)
 {
@@ -317,13 +358,17 @@ gtv_finalise_dbus (gtk_video_private_t *priv)
 }
 #else
 #define screensaver_is_running_dbus(PRIV) (FALSE)
+#define gtv_screensaver_maybe_inhibit_dbus(PRIV, INHIBIT)
+#define gtv_init_dbus(PRIV)
 #define gtv_finalise_dbus(PRIV)
 #endif
 
-GtkType gtk_video_get_type (void) {
+GtkType gtk_video_get_type (void)
+{
   static GtkType gtk_video_type = 0;
 
-  if (!gtk_video_type) {
+  if (!gtk_video_type)
+  {
     static const GTypeInfo gtk_video_info = {
       sizeof (GtkVideoClass),
       (GBaseInitFunc) NULL,
@@ -335,7 +380,7 @@ GtkType gtk_video_get_type (void) {
       0 /* n_preallocs */,
       (GInstanceInitFunc) gtk_video_instance_init,
     };
-    
+
     gtk_video_type = g_type_register_static (GTK_TYPE_WIDGET,
 					     "GtkVideo", &gtk_video_info, (GTypeFlags)0);
   }
@@ -407,8 +452,8 @@ static gboolean key_snoop_cb (GtkWidget *w, GdkEventKey *e, gpointer d)
 
 #endif
 
-static void gtk_video_class_init (GtkVideoClass *class) {
-
+static void gtk_video_class_init (GtkVideoClass *class)
+{
   GObjectClass    *object_class;
   GtkWidgetClass  *widget_class;
 
@@ -424,7 +469,7 @@ static void gtk_video_class_init (GtkVideoClass *class) {
   widget_class->size_allocate = gtk_video_size_allocate;
   /*widget_class->size_request  = gtk_video_size_request;*/
   widget_class->expose_event  = (typeof (widget_class->expose_event)) gtk_true;
-  
+
   /* GObject */
   object_class->set_property  = NULL;
   object_class->get_property  = NULL;
@@ -502,10 +547,13 @@ static void dest_size_cb (void *gv_gen,
   GtkVideo            *gv = (GtkVideo *) gv_gen;
   gtk_video_private_t *priv = gv->priv;
 
-  if (is_fullscreen (priv)) {
+  if (is_fullscreen (priv))
+  {
     *dest_width  = priv->fullscreen.width;
     *dest_height = priv->fullscreen.height;
-  } else {
+  }
+  else
+  {
     *dest_width  = gv->widget.allocation.width;
     *dest_height = gv->widget.allocation.height;
   }
@@ -601,12 +649,15 @@ static void frame_output_cb (void *gv_gen,
     first = FALSE;
   }
 
-  if (is_fullscreen (priv)) {
+  if (is_fullscreen (priv))
+  {
     *win_x = 0;
     *win_y = 0;
     *dest_width  = priv->fullscreen.width;
     *dest_height = priv->fullscreen.height;
-  } else {
+  }
+  else
+  {
     *win_x = priv->pos.width;
     *win_y = priv->pos.height;
     *dest_width  = gtv->widget.allocation.width;
@@ -616,8 +667,8 @@ static void frame_output_cb (void *gv_gen,
   *dest_pixel_aspect = priv->display_ratio;
 }
 
-static xine_video_port_t *load_video_out_driver(GtkVideo *this) {
-
+static xine_video_port_t *load_video_out_driver (GtkVideo *this)
+{
   double                   res_h, res_v;
   x11_visual_t             vis;
   const char              *video_driver_id;
@@ -633,9 +684,8 @@ static xine_video_port_t *load_video_out_driver(GtkVideo *this) {
 			   / DisplayHeightMM (vis.display, vis.screen));
   priv->display_ratio   = res_v / res_h;
 
-  if (fabs(priv->display_ratio - 1.0) < 0.01) {
+  if (fabs(priv->display_ratio - 1.0) < 0.01)
     priv->display_ratio   = 1.0;
-  }
 
   vis.dest_size_cb      = dest_size_cb;
   vis.frame_output_cb   = frame_output_cb;
@@ -643,31 +693,19 @@ static xine_video_port_t *load_video_out_driver(GtkVideo *this) {
 
   if (priv->video_driver_id)
     video_driver_id = priv->video_driver_id;
-  else {
-
-    char **driver_ids = (char **) xine_list_video_output_plugins (priv->xine) ;
-    char **choices;
-    int i;
-
-    choices = malloc (sizeof (char *) * 100);
-    choices[0] = "auto"; i=0;
-    while (driver_ids[i]) {
-      choices[i+1] = strdup(driver_ids[i]);
-      i++;
-    }
-    choices[i+1]=0;
-    
-
+  else
+  {
+    char **choices = get_driver_ids (xine_list_video_output_plugins (priv->xine));
     /* try to init video with stored information */
-    i = xine_config_register_enum (priv->xine,
+    int i = xine_config_register_enum (priv->xine,
 				   "video.driver", 0,
 				   choices,
 				   N_("video driver to use"),
 				   NULL, 10, NULL, NULL);
     video_driver_id = choices[i];
   }
-  if (strcmp (video_driver_id, "auto")) {
-
+  if (strcmp (video_driver_id, "auto"))
+  {
     video_port=xine_open_video_driver (priv->xine,
 				      video_driver_id,
 				      XINE_VISUAL_TYPE_X11,
@@ -719,7 +757,8 @@ static GdkFilterReturn xevent_filter_cb (XEvent *const xev, GdkEvent *const ev,
   if (xev->xany.window != GDK_WINDOW_XWINDOW (gtv->widget.window))
     return GDK_FILTER_CONTINUE;
 
-  switch (xev->type) {
+  switch (xev->type)
+  {
   case Expose:
     if (!xev->xexpose.count)
       xine_port_send_gui_data (priv->video_port,
@@ -737,7 +776,7 @@ static GdkFilterReturn xevent_filter_cb (XEvent *const xev, GdkEvent *const ev,
 
   case MotionNotify:
     logprintf ("gtkvideo: mouse event: mx=%d my=%d\n",
-	       xev->xmotion.x, xev->xmotion.y); 
+	       xev->xmotion.x, xev->xmotion.y);
     gtv_send_xine_mouse_event (priv, xev->xmotion.x, xev->xmotion.y, 0);
     if (priv->fullscreen_mode)
       gtv_show_pointer (gtv);
@@ -843,9 +882,7 @@ static void show_cb (GtkWidget *w, gpointer d)
 
 static gboolean gtv_screen_blanker_cb (GtkVideo *gtv)
 {
-  if (is_fullscreen (gtv->priv) && !playlist_showing_logo ()
-      && xine_get_param (gtv->priv->stream, XINE_PARAM_SPEED))
-    gtv_unblank_screen (gtv);
+  gtv_unblank_screen (gtv);
   return TRUE;
 }
 
@@ -978,7 +1015,8 @@ static void gtk_video_realize (GtkWidget *widget)
   GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
 
   display = gdk_display_open (gdk_display_get_name (gdk_display_get_default ()));
-  if (!display) {
+  if (!display)
+  {
     g_printerr (_("gtkvideo: XOpenDisplay failed!\n"));
     return;
   }
@@ -1058,11 +1096,12 @@ static void gtk_video_realize (GtkWidget *widget)
    * load audio, video drivers
    */
 
-  priv->video_port = load_video_out_driver (this) ;
+  priv->video_port = load_video_out_driver (this);
 
-  if (!priv->video_port) {
+  if (!priv->video_port)
+  {
     g_printerr (_("gtkvideo: couldn't open video driver\n"));
-    return ;
+    return;
   }
 
   /*
@@ -1077,18 +1116,20 @@ static void gtk_video_realize (GtkWidget *widget)
   priv->pointer[1] = XCreateFontCursor (xdisplay, XC_left_ptr);
   priv->pointer_visible = TRUE;
   XUnlockDisplay (xdisplay);
+
+  priv->screen_blanker_timeout =
+    g_timeout_add (4000, (GSourceFunc) gtv_screen_blanker_cb, gtv);
 }
 
 
-xine_video_port_t *gtk_video_get_port  (GtkVideo *gtv) {
-  gtk_video_private_t *priv;
-
-  priv = gtv->priv;
-
+xine_video_port_t *gtk_video_get_port (GtkVideo *gtv)
+{
+  gtk_video_private_t *priv = gtv->priv;
   return priv->video_port;
 }
 
-static void gtk_video_unrealize (GtkWidget *widget) {
+static void gtk_video_unrealize (GtkWidget *widget)
+{
   GtkVideo            *this;
   gtk_video_private_t *priv;
 
@@ -1098,6 +1139,12 @@ static void gtk_video_unrealize (GtkWidget *widget) {
   this = (GtkVideo *) widget;
   priv = this->priv;
 
+  if (priv->screen_blanker_timeout)
+  {
+    g_source_remove (priv->screen_blanker_timeout);
+    priv->screen_blanker_timeout = 0;
+  }
+
   /* stop event thread */
   if (priv->thread)
     pthread_cancel (priv->thread);
@@ -1158,14 +1205,12 @@ GtkWidget *gtk_video_new (xine_t *xine, xine_stream_t *stream,
   else
     priv->video_driver_id = NULL;
 
-#ifdef WITH_DBUS
   gtv_init_dbus (gtv);
-#endif
 
   return this;
 }
 
-static void gtk_video_size_allocate (GtkWidget *widget, 
+static void gtk_video_size_allocate (GtkWidget *widget,
     				     GtkAllocation *allocation)
 {
   GtkVideo *this;
@@ -1270,7 +1315,7 @@ void gtk_video_rescale (GtkVideo *gtv, double scale)
     double factor = DOUBLE (scale, priv->video_size.width,
 				  priv->video_size.height);
     priv->size = SCALE (factor);
-    priv->resize_factor = scale; 
+    priv->resize_factor = scale;
     logprintf ("rescale: size = %d,%d factor=%f\n",
 	       priv->size.width, priv->size.height, scale);
     if (is_fullscreen (priv) || !GTK_WIDGET_VISIBLE (&gtv->widget))
@@ -1285,7 +1330,7 @@ static gboolean unblock_cb (gpointer gtv)
 {
   logprintf ("unblocking rescale\n");
   ((GtkVideo *)gtv)->priv->block_next_scale_change = FALSE;
-  ((GtkVideo *)gtv)->priv->unblock_timeout = 0;  
+  ((GtkVideo *)gtv)->priv->unblock_timeout = 0;
   return FALSE;
 }
 
@@ -1325,7 +1370,7 @@ void gtk_video_set_fullscreen (GtkVideo *gtv, gint fullscreen)
   if (fullscreen == is_fullscreen (priv))
     return;
 
-  pthread_mutex_lock (&engine_lock);
+  g_static_rec_mutex_lock (&engine_lock);
   display = gtv_get_xdisplay (gtv);
   XLockDisplay (display);
   toplevel = GTK_WINDOW (gtk_widget_get_toplevel (&gtv->widget));
@@ -1342,12 +1387,7 @@ void gtk_video_set_fullscreen (GtkVideo *gtv, gint fullscreen)
     do_pending_events (); /* events have been generated - process them */
     gtk_video_get_fullscreen_geometry (gtv);
     gtv_hide_pointer (gtv);
-    priv->screen_blanker_timeout =
-      g_timeout_add (4000, (GSourceFunc) gtv_screen_blanker_cb, gtv);
-#ifdef WITH_DBUS
-    if (screensaver_is_running_dbus (gtv->priv))
-      gtv_screensaver_inhibit_dbus (gtv->priv, TRUE);
-#endif
+    gtv_screensaver_maybe_inhibit_dbus (gtv->priv, TRUE);
   }
   else
   {
@@ -1359,15 +1399,7 @@ void gtk_video_set_fullscreen (GtkVideo *gtv, gint fullscreen)
     gdk_flush ();
     /* just unfullscreen it - beyond that, rely on window-state-event */
     gtk_window_unfullscreen (toplevel);
-    if (priv->screen_blanker_timeout)
-    {
-      g_source_remove (priv->screen_blanker_timeout);
-      priv->screen_blanker_timeout = 0;
-    }
-#ifdef WITH_DBUS
-    if (screensaver_is_running_dbus (gtv->priv))
-      gtv_screensaver_inhibit_dbus (gtv->priv, FALSE);
-#endif
+    gtv_screensaver_maybe_inhibit_dbus (gtv->priv, gtv_test_inhibit_blank (gtv));
     gtv_show_pointer (gtv);
     gdk_flush ();
     sched_yield (); /* hmm, seems to be needed here... */
@@ -1384,33 +1416,49 @@ void gtk_video_set_fullscreen (GtkVideo *gtv, gint fullscreen)
   }
 
   XUnlockDisplay (display);
-  pthread_mutex_unlock (&engine_lock);
+  g_static_rec_mutex_unlock (&engine_lock);
 }
 
-gint gtk_video_is_fullscreen (GtkVideo *gtv) {
-
+gint gtk_video_is_fullscreen (GtkVideo *gtv)
+{
   g_return_val_if_fail (gtv != NULL, 0);
   g_return_val_if_fail (GTK_IS_VIDEO (gtv), 0);
 
   return !!gtv->priv->fullscreen_mode;
 }
 
-void gtk_video_set_auto_resize (GtkVideo *gtv, gboolean resize) {
-
+void gtk_video_set_auto_resize (GtkVideo *gtv, gboolean resize)
+{
   g_return_if_fail (gtv != NULL);
   g_return_if_fail (GTK_IS_VIDEO (gtv));
 
   gtv->priv->auto_resize = resize;
 }
 
-gboolean gtk_video_get_auto_resize (GtkVideo *gtv) {
-
+gboolean gtk_video_get_auto_resize (GtkVideo *gtv)
+{
   g_return_val_if_fail (gtv != NULL, 0);
   g_return_val_if_fail (GTK_IS_VIDEO (gtv), 0);
 
   return gtv->priv->auto_resize;
 }
 
+void gtk_video_set_windowed_unblank (GtkVideo *gtv, gboolean unblank)
+{
+  g_return_if_fail (gtv != NULL);
+  g_return_if_fail (GTK_IS_VIDEO (gtv));
+
+  gtv->priv->playing_no_blank = unblank;
+}
+
+gboolean gtk_video_get_windowed_unblank (GtkVideo *gtv)
+{
+  g_return_val_if_fail (gtv != NULL, 0);
+  g_return_val_if_fail (GTK_IS_VIDEO (gtv), 0);
+
+  return gtv->priv->playing_no_blank;
+}
+
 void
 gtk_video_deny_shrink (GtkVideo *gtv)
 {
@@ -1435,7 +1483,7 @@ gtv_allow_shrink (gpointer gtv)
 {
   if (is_fullscreen (((GtkVideo *)gtv)->priv))
     return FALSE;
-  
+
   ((GtkVideo *)gtv)->priv->minimising = 0;
   ((GtkVideo *)gtv)->widget.requisition =
     (GtkRequisition){ GTK_VIDEO_MIN_WIDTH, GTK_VIDEO_MIN_HEIGHT };
@@ -1460,10 +1508,14 @@ gtk_video_allow_shrink (GtkVideo *gtv)
 static GList *gtv_create_plugins (gtk_video_private_t *priv,
 				  xine_audio_port_t **audio,
 				  xine_video_port_t **video,
-				  const char *plugins, int type)
+				  const char *plugins, int type,
+				  gboolean is_deinterlace)
 {
   GList *list = NULL;
 
+  if (is_deinterlace)
+    priv->have_tvtime = FALSE;
+
   if (!plugins || !plugins[0])
     return NULL;
 
@@ -1480,13 +1532,15 @@ static GList *gtv_create_plugins (gtk_video_private_t *priv,
       if (args > next)
 	args = NULL;
 
-      logprintf ("plugin: %p %.*s\n", plugins, (args?:next) - plugins, plugins);
+      logprintf ("plugin: %p %.*s\n", plugins, (int)((args?:next) - plugins), plugins);
       if (args)
 	logprintf ("args  : %p %.*s\n", args ? args + 1 : NULL,
-		   args ? next - args - 1 : 0, args ? args + 1 : NULL);
+		   (int)(args ? next - args - 1 : 0), args ? args + 1 : NULL);
       logprintf ("next  : %p\n", next);
 
       name = g_strndup (plugins, (args ? : next) - plugins);
+      if (is_deinterlace && !g_strcasecmp (name, "tvtime"))
+	priv->have_tvtime = TRUE;
       plugin = xine_post_init (priv->xine, name, 0, audio, video);
       if (plugin)
       {
@@ -1622,7 +1676,7 @@ void gtk_video_set_post_plugins_video (GtkVideo *gtv, const char *params)
   gtv_dispose_plugins (priv, priv->post_video.list);
   priv->post_video.list =
     gtv_create_plugins (priv, NULL, &priv->video_port, params,
-			XINE_POST_TYPE_VIDEO_FILTER);
+			XINE_POST_TYPE_VIDEO_FILTER, FALSE);
   gtv_wire_plugins_video (priv);
 }
 
@@ -1719,7 +1773,7 @@ void gtk_video_set_post_plugins_audio (GtkVideo *gtv, const char *params,
   gtv_dispose_plugins (priv, priv->post_audio.list);
   priv->post_audio.list =
     gtv_create_plugins (priv, &audio_port, NULL, params,
-			XINE_POST_TYPE_AUDIO_FILTER);
+			XINE_POST_TYPE_AUDIO_FILTER, FALSE);
   gtv_wire_plugins_audio (priv, audio_port);
 }
 
@@ -1855,13 +1909,13 @@ void gtk_video_set_post_plugins_deinterlace (GtkVideo *gtv, const char *params)
   gtv_dispose_plugins (priv, priv->post_deinterlace.list);
   priv->post_deinterlace.list =
     gtv_create_plugins (priv, NULL, &priv->video_port, params,
-			XINE_POST_TYPE_VIDEO_FILTER);
+			XINE_POST_TYPE_VIDEO_FILTER, TRUE);
   gtv_wire_plugins_video (priv);
 
   /* allow the old method if there are no created deinterlace plugins */
   xine_set_param (priv->stream, XINE_PARAM_VO_DEINTERLACE,
 		  priv->post_deinterlace.enable &&
-		    !priv->post_deinterlace.list);
+		    (!priv->post_deinterlace.list || priv->have_tvtime));
 }
 
 void gtk_video_set_use_post_plugins_deinterlace (GtkVideo *gtv, gboolean on)
@@ -1879,7 +1933,7 @@ void gtk_video_set_use_post_plugins_deinterlace (GtkVideo *gtv, gboolean on)
 
   /* allow the old method if there are no created deinterlace plugins */
   xine_set_param (priv->stream, XINE_PARAM_VO_DEINTERLACE,
-		  on && !priv->post_deinterlace.list);
+		  on && (!priv->post_deinterlace.list || priv->have_tvtime));
 }
 
 gboolean gtk_video_get_use_post_plugins_deinterlace (GtkVideo *gtv)
@@ -1909,16 +1963,52 @@ gboolean gtk_video_get_auto_rescale (GtkVideo *gtv)
   return (gtv && gtv->priv) ? gtv->priv->auto_rescale : FALSE;
 }
 
+static int screensaver_poke (const char *const argv[], const char *name)
+{
+  int ret = 60; /* default return = try again in 4 minutes */
+  GError *err = NULL;
+
+  g_spawn_async (NULL, (gchar **)argv, NULL,
+		 G_SPAWN_SEARCH_PATH | G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
+		 NULL, NULL, NULL, &err);
+  if (err)
+  {
+    g_printerr (_("gtkvideo: can't poke %s: %s\n"), name, err->message);
+    if (err->domain == G_SPAWN_ERROR)
+      switch (err->code)
+      {
+      case G_SPAWN_ERROR_ACCES:
+      case G_SPAWN_ERROR_PERM:
+      case G_SPAWN_ERROR_NOEXEC:
+      case G_SPAWN_ERROR_NOENT:
+      case G_SPAWN_ERROR_ISDIR:
+	g_printerr (_("gtkvideo: not trying again\n"));
+	ret = -1;
+	break;
+      default:
+	ret = 60;
+	break;
+      }
+    g_error_free (err);
+  }
+
+  return ret;
+}
+
 static gboolean gtv_unblank_screen (GtkVideo *gtv)
 {
-  gtk_video_private_t *priv = gtv->priv;
-  Display *display;
-  static int gnome_screensaver = 0;
+  static gboolean dbus_deny = FALSE;
 
-  if (!is_fullscreen (priv))
+  if (!gtv_test_inhibit_blank (gtv))
+  {
+    if (dbus_deny)
+      gtv_screensaver_maybe_inhibit_dbus (gtv->priv, dbus_deny = FALSE);
     return FALSE;
+  }
+  else if (!dbus_deny)
+    gtv_screensaver_maybe_inhibit_dbus (gtv->priv, dbus_deny = TRUE);
 
-  display = gtv_get_xdisplay (gtv);
+  Display *display = gtv_get_xdisplay (gtv);
   XLockDisplay (display);
 #ifdef HAVE_DPMS_EXTENSION
   union { int i; CARD16 c; } dummy;
@@ -1928,12 +2018,13 @@ static gboolean gtv_unblank_screen (GtkVideo *gtv)
     DPMSForceLevel (display, DPMSModeOn);
 #endif
 #ifdef HAVE_XTESTEXTENSION
+  gtk_video_private_t *priv = gtv->priv;
   if (priv->have_xtest == True)
   {
     static unsigned char flip = 0;
     int state = !(shift & ((flip = !flip) + 1));
     XTestFakeKeyEvent (display, priv->kc_shift[flip], state, CurrentTime);
-    XTestFakeKeyEvent (display, priv->kc_shift[flip], !state, CurrentTime); 
+    XTestFakeKeyEvent (display, priv->kc_shift[flip], !state, CurrentTime);
     XSync (display, False);
   }
   else
@@ -1942,40 +2033,32 @@ static gboolean gtv_unblank_screen (GtkVideo *gtv)
     XForceScreenSaver (display, ScreenSaverReset);
   }
 
-  if (!screensaver_is_running_dbus (priv))
+  if (!screensaver_is_running_dbus (gtv->priv))
   {
+#ifdef GNOME_SCREENSAVER_COMMAND
     /* poke GNOME screensaver (assume that $PATH is sane) */
+    static int gnome_screensaver = 0;
     if (gnome_screensaver > 0)
       --gnome_screensaver;
     if (!gnome_screensaver)
     {
-      GError *err = NULL;
-      const char *const argv[] = { GNOME_SCREENSAVER_COMMAND, "--poke", NULL };
-      g_spawn_async (NULL, (gchar **)argv, NULL,
-		     G_SPAWN_SEARCH_PATH | G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
-		     NULL, NULL, NULL, &err);
-      if (err)
-      {
-	g_printerr (_("gtkvideo: can't poke gnome-screensaver: %s\n"),
-		 err->message);
-	if (err->domain == G_SPAWN_ERROR)
-	  switch (err->code)
-	  {
-	  case G_SPAWN_ERROR_ACCES:
-	  case G_SPAWN_ERROR_PERM:
-	  case G_SPAWN_ERROR_NOEXEC:
-	  case G_SPAWN_ERROR_NOENT:
-	  case G_SPAWN_ERROR_ISDIR:
-	    g_printerr (_("gtkvideo: not trying again\n"));
-	    gnome_screensaver = -1;
-	    break;
-	  default:
-	    gnome_screensaver = 60; /* try again in 4 minutes */
-	    break;
-	}
-	g_error_free (err);
-      }
+      static const char *const argv[] = { GNOME_SCREENSAVER_COMMAND, "--poke", NULL };
+      /* screensaver name is used in "can't poke <name>" */
+      gnome_screensaver = screensaver_poke (argv, _("gnome-screensaver"));
     }
+#endif
+
+#ifdef X_SCREENSAVER_COMMAND
+    /* similarly, poke xscreensaver */
+    static int x_screensaver = 0;
+    if (x_screensaver > 0)
+      --x_screensaver;
+    if (!x_screensaver)
+    {
+      static const char *const argv[] = { X_SCREENSAVER_COMMAND, "--deactivate", NULL };
+      x_screensaver = screensaver_poke (argv, _("xscreensaver"));
+    }
+#endif
   }
 
   XUnlockDisplay (display);
