@@ -13,9 +13,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: gtkvideo.h,v 1.10.2.1 2005/05/31 22:13:29 dsalt Exp $
+ * $Id: gtkvideo.h,v 1.23 2006/04/04 21:58:54 dsalt Exp $
  *
  * a gtk xine video widget
  */
@@ -34,12 +34,14 @@ extern "C" {
 
 /* Signals */
 enum {
-  GTK_VIDEO_KEYPRESS,	/* caught keypress in fullscreen mode */
-  GTK_VIDEO_KEYRELEASE,	/* caught keyrelease in fullscreen mode */
+  GTK_VIDEO_SCALE_FACTOR, /* video scale has been changed by a resize */
   LAST_SIGNAL
 };
 
 
+#define GTK_VIDEO_MIN_WIDTH 32
+#define GTK_VIDEO_MIN_HEIGHT 24
+
 #define GTK_VIDEO(obj)             (GTK_CHECK_CAST ((obj), gtk_video_get_type (), GtkVideo))
 #define GTK_VIDEO_CLASS(klass)     (GTK_CHECK_CLASS_CAST ((klass), gtk_video_get_type (), GtkVideoClass))
 #define GTK_IS_VIDEO(obj)          (GTK_CHECK_TYPE (obj, gtk_video_get_type ()))
@@ -50,7 +52,7 @@ enum {
 
   typedef struct gtk_video_private_s gtk_video_private_t;
 
-  
+
   struct _GtkVideo
   {
     GtkWidget                widget;
@@ -60,12 +62,13 @@ enum {
   struct _GtkVideoClass
   {
     GtkWidgetClass parent_class;
-    void (*keypress) (GtkWidget *gtv);
-    void (*keyrelease) (GtkWidget *gtv);
+    void (*keypress) (GtkWidget *, GdkEventKey *, gpointer);
+    void (*keyrelease) (GtkWidget *, GdkEventKey *, gpointer);
+    void (*scale_factor) (GtkWidget *, double, gpointer);
   };
 
   GtkType    gtk_video_get_type          (void);
-  GtkWidget* gtk_video_new               (xine_t *xine, xine_stream_t *stream, 
+  GtkWidget* gtk_video_new               (xine_t *xine, xine_stream_t *stream,
       					  xine_post_out_t *post_out, /* will be rewired */
       					  const gchar *video_driver_id,
 					  int default_width, int default_height,
@@ -77,41 +80,44 @@ enum {
 		 			  gint x, gint y,
 		 			  gint width,
 					  gint height);
-  void       gtk_video_rescale           (GtkVideo *gtv, float scale);
+  void       gtk_video_rescale           (GtkVideo *gtv, double scale);
   xine_video_port_t *gtk_video_get_port  (GtkVideo *gtv);
-  gboolean gtk_video_is_fullscreen (GtkVideo *gtv) ;
+  void gtk_video_reshow (GtkVideo *);
+  gboolean gtk_video_is_fullscreen (GtkVideo *gtv);
   void gtk_video_set_fullscreen (GtkVideo *gtv, gboolean fullscreen);
-  gboolean gtk_video_get_auto_resize (GtkVideo *gtv) ;
+    /* note: up to the caller to make sticky etc.; use window-state-event */
+  void gtk_video_make_transient_for_fullscreen (GtkVideo *gtv, GtkWindow *);
+  const GtkRequisition *gtk_video_get_fullscreen_geometry (GtkVideo *gtv);
+  gboolean gtk_video_get_auto_resize (GtkVideo *gtv);
   void gtk_video_set_auto_resize (GtkVideo *gtv, gboolean resize);
-
-  void gtk_video_grab_focus (GtkVideo *gtv);
+  gboolean gtk_video_get_windowed_unblank (GtkVideo *gtv);
+  void gtk_video_set_windowed_unblank (GtkVideo *gtv, gboolean unblank);
+  void gtk_video_deny_shrink (GtkVideo *gtv);
+  gboolean gtk_video_allow_shrink (GtkVideo *gtv);
 
   gboolean gtk_video_select_vis (GtkVideo *, const char *id,
 				 xine_audio_port_t **);
   gboolean gtk_video_set_vis (GtkVideo *, xine_audio_port_t **, gboolean);
 
+  void gtk_video_set_post_plugins_deinterlace (GtkVideo *, const char *);
   void gtk_video_set_post_plugins_video (GtkVideo *, const char *);
   void gtk_video_set_post_plugins_audio (GtkVideo *, const char *,
 					 xine_audio_port_t *);
+  void gtk_video_set_use_post_plugins_deinterlace (GtkVideo *, gboolean);
   void gtk_video_set_use_post_plugins_video (GtkVideo *, gboolean);
   void gtk_video_set_use_post_plugins_audio (GtkVideo *, gboolean,
 					     xine_audio_port_t *);
+  gboolean gtk_video_get_use_post_plugins_deinterlace (GtkVideo *);
   gboolean gtk_video_get_use_post_plugins_video (GtkVideo *);
   gboolean gtk_video_get_use_post_plugins_audio (GtkVideo *);
 
-  void gtk_video_set_tvtime (GtkVideo *, const char *);
-  void gtk_video_set_deinterlace (GtkVideo *, gboolean);
-  gboolean gtk_video_get_deinterlace (GtkVideo *);
-  xine_post_t *gtk_video_get_deinterlacer (GtkVideo *);
-
   void gtk_video_set_auto_rescale (GtkVideo *, gboolean);
   gboolean gtk_video_get_auto_rescale (GtkVideo *);
 
+  void gtk_video_unblank_screen (GtkVideo *);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
-
-
 #endif /* HAVE_GTK_VIDEO_H */
-
