@@ -1,4 +1,4 @@
-/* 
+/*
  * Copyright (C) 2003-2006 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
@@ -67,9 +67,11 @@ typedef struct {
   guint        keyval;
   guint        state;
   const gchar *keyname;
+  gboolean     std;
 } key_binding_t;
 
 static key_binding_t editkey;
+static const char *editdesc;
 static const key_binding_t null_binding = { 0, 0, GDK_VoidSymbol, 0 };
 static key_binding_t default_binding = { N_("New binding"), "", GDK_VoidSymbol, 0 };
 
@@ -78,11 +80,16 @@ static GtkTreeIter catch_key_iter;
 static void delete_key_binding (GtkTreeIter *);
 static gchar *kb2str (key_binding_t *kb);
 
+#define COLUMN_DESC 0
+#define COLUMN_KEY  1
+#define COLUMN_DATA 2
+
 static const key_binding_t default_bindings[] = {
   { N_("Play"), "vdr ('PLAY') || play ();", GDK_Return, 0 },
   { N_("Exit gxine"), "exit ();", GDK_q, 0 },
   { N_("Pause"), "if (!vdr ('PAUSE') && !is_live_stream ()) pause ();", GDK_space, 0 },
   { N_("Stop"), "vdr ('STOP') || stop ();", 0, 0 },
+  { N_("Eject"), "eject ();", 0, 0 },
   { N_("Windowed mode"), "vo_fullscreen.v = false;", GDK_Escape, 0 },
   { N_("Fullscreen mode"), "vo_fullscreen.toggle ();", GDK_f, 0 },
   { N_("Aspect ratio"), "++vo_aspect.v;", GDK_a, 0 },
@@ -131,6 +138,8 @@ static const key_binding_t default_bindings[] = {
   { N_("Back"), "vdr ('BACK');", GDK_BackSpace, 0 },
   /* (VDR) Audio menu */
   { N_("Audio"), "vdr ('AUDIO');", 0, 0 },
+  /* (VDR) Info display */
+  { N_("Info"), "vdr ('INFO');", 0, 0 },
   { N_("Playlist next"), "playlist_play (playlist_get_item()+1);", GDK_Page_Down, 0 },
   { N_("Playlist previous"), "playlist_play (playlist_get_item()-1);", GDK_Page_Up, 0 },
   { N_("Zoom in"), "vo_zoom.v += 5;", GDK_Z, GDK_SHIFT_MASK },
@@ -142,27 +151,52 @@ static const key_binding_t default_bindings[] = {
   { 0 }
 };
 
+static inline const char *getdesc (const key_binding_t *binding)
+{
+   return binding->std ? gettext (binding->desc) : binding->desc;
+}
 
 static key_binding_t *lookup_binding (GtkTreeIter *iter)
 {
   key_binding_t *k;
   GValue v = {0};
 
-  gtk_tree_model_get_value (kb_model, iter, 2, &v);
+  gtk_tree_model_get_value (kb_model, iter, COLUMN_DATA, &v);
   k = g_value_peek_pointer (&v);
   g_value_unset (&v);
   return k;
 }
 
 
+static gboolean is_menu_accel (GtkWidget *w, const GdkEventKey *event)
+{
+  guint keyval = 0;
+  GdkModifierType mods = 0;
+  gchar *accelstr;
+  g_object_get (gtk_widget_get_settings (w), "gtk-menu-bar-accel",
+		&accelstr, NULL);
+  if (accelstr)
+  {
+    gtk_accelerator_parse (accelstr, &keyval, &mods);
+    free (accelstr);
+  }
+  if (!keyval)
+    keyval = GDK_F10; /* default binding */
+
+  return event->keyval == keyval &&
+	 (event->state & GXINE_MODIFIER_MASK) == (mods & GXINE_MODIFIER_MASK);
+}  
+
 gboolean keypress_cb (GtkWidget *win, GdkEventKey *event, gpointer data)
 {
   GtkTreeIter iter;
   guint keyval = gdk_keyval_to_lower (event->keyval);
   guint state = event->state & GXINE_MODIFIER_MASK;
 
-  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(kb_store), &iter)) {
-    do {
+  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(kb_store), &iter))
+  {
+    do
+    {
       key_binding_t *key_binding = lookup_binding (&iter);
       if (key_binding->state == state &&
 	  (key_binding->keyval == keyval ||
@@ -184,7 +218,7 @@ gboolean keypress_cb (GtkWidget *win, GdkEventKey *event, gpointer data)
        || gtk_bindings_activate_event (data, event)))
     return TRUE;
 
-  /* Windowed-mode separate toolbar (Alt-<key>, F10, Menu) */
+  /* Windowed-mode separate toolbar (Alt-<key>, GTK menu bar binding, Menu) */
   if (win == app && menubar)
   {
     GtkWidget *toplevel = gtk_widget_get_toplevel (menubar);
@@ -197,17 +231,19 @@ gboolean keypress_cb (GtkWidget *win, GdkEventKey *event, gpointer data)
     }
   }
 
-  if (state == 0)
-    switch (keyval)
-    {
-    case GDK_F10:
-      if (menubar && GTK_WIDGET_VISIBLE (gtk_widget_get_toplevel (menubar)))
-	break;
-    case GDK_Menu:
-      gtk_menu_popup (GTK_MENU(popup_menu), NULL, NULL, NULL, NULL, 0,
-                      gtk_get_current_event_time ());
-      return TRUE;
-    }
+  if (keyval == GDK_Menu && state == 0)
+  {
+    do_popup:
+    gtk_menu_popup (GTK_MENU(popup_menu), NULL, NULL, NULL, NULL, 0,
+		    gtk_get_current_event_time ());
+    return TRUE;
+  }
+  else if (is_menu_accel (win, event))
+  {
+    GtkWidget *toplevel = gtk_widget_get_toplevel (menubar);
+    if (!menubar || !GTK_WIDGET_VISIBLE (toplevel == app ? menubar : toplevel))
+      goto do_popup;
+  }
 
   logprintf ("key_events: key %d not bound\n", event->keyval);
 
@@ -260,7 +296,8 @@ static void close_edit_window (void)
 }
 
 
-static gboolean close_cb (GtkWidget* widget, gpointer data) {
+static gboolean close_cb (GtkWidget* widget, gpointer data)
+{
   close_edit_window ();
   close_list_window ();
   return TRUE;
@@ -273,9 +310,10 @@ static key_binding_t *find_key_binding (const gchar *desc, const gchar *cmd,
   if (!desc && !cmd)
     return NULL;
 
-  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(kb_store), iter)) {
-
-    do {
+  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(kb_store), iter))
+  {
+    do
+    {
       key_binding_t *key_binding = lookup_binding (iter);
 
       /* allow either to be null, in which case we match on the other rather
@@ -291,21 +329,24 @@ static key_binding_t *find_key_binding (const gchar *desc, const gchar *cmd,
   return NULL;
 }
 
-static gchar* kb2str (key_binding_t *kb) {
-
-  static const gchar *text_shift = NULL, *text_control,
-		     *text_mod1, *text_mod3, *text_mod4;
+static gchar* kb2str (key_binding_t *kb)
+{
+  static struct {
+    const gchar *shift, *ctrl, *alt, *mod2, *mod3, *mod4, *mod5;
+  } text = { NULL };
   guint              l;
   gchar             *name;
   const gchar       *key;
 
-  if (!text_shift)
+  if (!text.shift)
   {
-    text_shift   = _("<Shift>");
-    text_control = _("<Control>");
-    text_mod1    = _("<Alt>");
-    text_mod3    = _("<Mod3>");
-    text_mod4    = _("<Mod4>");
+    text.shift = _("<Shift>");
+    text.ctrl  = _("<Control>");
+    text.alt   = _("<Alt>");
+    text.mod2  = _("<Mod2>");
+    text.mod3  = _("<Mod3>");
+    text.mod4  = _("<Mod4>");
+    text.mod5  = _("<Mod5>");
   }
 
   /*
@@ -314,15 +355,19 @@ static gchar* kb2str (key_binding_t *kb) {
 
   l = 1;
   if (kb->state & GDK_SHIFT_MASK)
-    l += strlen (text_shift);
+    l += strlen (text.shift);
   if (kb->state & GDK_CONTROL_MASK)
-    l += strlen (text_control);
+    l += strlen (text.ctrl);
   if (kb->state & GDK_MOD1_MASK)
-    l += strlen (text_mod1);
+    l += strlen (text.alt);
+  if (kb->state & GDK_MOD2_MASK)
+    l += strlen (text.mod2);
   if (kb->state & GDK_MOD3_MASK)
-    l += strlen (text_mod3);
+    l += strlen (text.mod3);
   if (kb->state & GDK_MOD4_MASK)
-    l += strlen (text_mod4);
+    l += strlen (text.mod4);
+  if (kb->state & GDK_MOD5_MASK)
+    l += strlen (text.mod5);
 
   if (kb->keyval == GDK_VoidSymbol)
     key = _("undef"); /* undefined */
@@ -334,15 +379,19 @@ static gchar* kb2str (key_binding_t *kb) {
   *name = 0;
 
   if (kb->state & GDK_SHIFT_MASK)
-    strcat (name, text_shift);
+    strcat (name, text.shift);
   if (kb->state & GDK_CONTROL_MASK)
-    strcat (name, text_control);
+    strcat (name, text.ctrl);
   if (kb->state & GDK_MOD1_MASK)
-    strcat (name, text_mod1);
+    strcat (name, text.alt);
+  if (kb->state & GDK_MOD2_MASK)
+    strcat (name, text.mod2);
   if (kb->state & GDK_MOD3_MASK)
-    strcat (name, text_mod3);
+    strcat (name, text.mod3);
   if (kb->state & GDK_MOD4_MASK)
-    strcat (name, text_mod4);
+    strcat (name, text.mod4);
+  if (kb->state & GDK_MOD5_MASK)
+    strcat (name, text.mod5);
 
   strcat (name, key);
 
@@ -352,9 +401,7 @@ static gchar* kb2str (key_binding_t *kb) {
 static void modify_key_binding (GtkTreeIter *iter, key_binding_t *key_binding,
 				const key_binding_t *new_binding)
 {
-  free ((char *)key_binding->keyname);
-  free ((char *)key_binding->cmd);
-  free ((char *)key_binding->desc);
+  key_binding_t old_binding = *key_binding;
 
   if (new_binding->keyval != GDK_VoidSymbol)
   {
@@ -364,6 +411,7 @@ static void modify_key_binding (GtkTreeIter *iter, key_binding_t *key_binding,
 
   key_binding->keyname = kb2str (key_binding);
   key_binding->cmd     = strdup (new_binding->cmd);
+
   if (!new_binding->desc)
   {
     int i = -1;
@@ -375,8 +423,19 @@ static void modify_key_binding (GtkTreeIter *iter, key_binding_t *key_binding,
   else
     key_binding->desc  = strdup (new_binding->desc);
 
-  gtk_list_store_set (kb_store, iter, 0, key_binding->desc, 1,
-		      key_binding->keyname, 2, key_binding, -1);
+  key_binding->std = new_binding->std &
+		     (!old_binding.desc ||
+		      !strcmp (old_binding.desc, key_binding->desc));
+
+  free ((char *)old_binding.keyname);
+  free ((char *)old_binding.cmd);
+  free ((char *)old_binding.desc);
+
+  gtk_list_store_set (kb_store, iter,
+		      COLUMN_DESC, getdesc (key_binding),
+		      COLUMN_KEY, key_binding->keyname,
+		      COLUMN_DATA, key_binding,
+		      -1);
 }
 
 
@@ -389,8 +448,8 @@ static void set_key_binding (const key_binding_t *new_binding)
 		? find_key_binding (new_binding->desc, NULL, &iter)
 		: find_key_binding (NULL, new_binding->cmd, &iter);
 
-  if (!key_binding) {
-
+  if (!key_binding)
+  {
     /* add new key binding */
 
     key_binding = g_malloc0 (sizeof (key_binding_t));
@@ -420,7 +479,7 @@ static void load_default_kb (void)
   while (default_bindings[++i].desc)
   {
     key_binding_t binding = default_bindings[i];
-    binding.desc = gettext (binding.desc);
+    binding.std = TRUE;
     set_key_binding (&binding);
   }
 }
@@ -428,27 +487,30 @@ static void load_default_kb (void)
 
 void save_key_bindings (void)
 {
-  gchar *fname = g_strconcat (g_get_home_dir(), "/.gxine/keybindings", NULL);
+  gchar *fname = get_config_filename (FILE_KEYBINDINGS);
   FILE *f = open_write (fname, _("Failed to save key bindings"));
-  
-  if (f) {
+
+  if (f)
+  {
     GtkTreeIter iter;
 
-    fprintf (f, "<GXINEKB VERSION=\"3\">\n");
+    fprintf (f, "<GXINEKB VERSION=\"4\">\n");
 
-    if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(kb_store), &iter)) {
-
-      do {
+    if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(kb_store), &iter))
+    {
+      do
+      {
 	key_binding_t *key_binding = lookup_binding (&iter);
 	char *desc = xml_escape_string (key_binding->desc, XML_ESCAPE_NO_QUOTE);
 	char *cmd = xml_escape_string (key_binding->cmd, XML_ESCAPE_NO_QUOTE);
 	fprintf (f, "  <KEYBINDING>\n"
-		    "    <DESCRIPTION>%s</DESCRIPTION>\n"
+		    "    <DESCRIPTION%s>%s</DESCRIPTION>\n"
 		    "    <COMMAND>%s</COMMAND>\n"
 		    "    <KEYVAL>%d</KEYVAL>\n"
 		    "    <STATE>%d</STATE>\n"
 		    "  </KEYBINDING>\n",
-		    desc, cmd, key_binding->keyval, key_binding->state);
+		    key_binding->std ? "" : " user='yes'", desc, cmd,
+		    key_binding->keyval, key_binding->state);
 	free (desc);
 	free (cmd);
       } while (gtk_tree_model_iter_next (kb_model, &iter));
@@ -530,16 +592,52 @@ static const kb_upgrade_t vdr_upgrade_to_0_5_0[] = {
   { NULL }
 };
 
+/* For upgrading from < 0.5.8 */
+static inline const char *
+xml2kb_to_std (const char *text)
+{
+  int i;
+  for (i = 0; default_bindings[i].desc; ++i)
+    if (!strcmp (text, gettext (default_bindings[i].desc)))
+      return default_bindings[i].desc;
+  return NULL;
+}
+
+/* For checking whether a description is standard (and translatable) */
+static inline gboolean
+xml2kb_is_std (const char *text)
+{
+  int i;
+  for (i = 0; default_bindings[i].desc; ++i)
+    if (!strcmp (text, default_bindings[i].desc))
+      return TRUE;
+  return FALSE;
+}
+
 static void xml2kb (xml_node_t *node, int version)
 {
   key_binding_t binding = { 0 };
-  int i;
 
   foreach_glist (node, node)
     if (!strcasecmp (node->name, "command"))
       binding.cmd = node->data;
     else if (!strcasecmp (node->name, "description"))
-      binding.desc = node->data;
+    {
+      if (version >= 4)
+      {
+	binding.std = !xml_parser_get_property (node, "user")
+		      && xml2kb_is_std (node->data);
+	binding.desc = node->data;
+      }
+      else
+      {
+	binding.desc = xml2kb_to_std (node->data);
+	binding.std = !!binding.desc;
+	if (!binding.desc)
+	  binding.desc = node->data;
+      }
+      binding.desc = strdup (binding.desc);
+    }
     else if (!strcasecmp (node->name, "keyval"))
       binding.keyval = atoi (node->data);
     else if (!strcasecmp (node->name, "state"))
@@ -553,16 +651,6 @@ static void xml2kb (xml_node_t *node, int version)
   if (version < 3)
     binding.cmd = kb_upgrade (binding.cmd, vdr_upgrade_to_0_5_0);
 
-  /* translate from English if a new translation is present and in use */
-  if (binding.desc)
-    for (i = 0; default_bindings[i].desc; ++i)
-      if (!strcmp (binding.desc, default_bindings[i].desc) &&
-	!strcmp (binding.cmd, default_bindings[i].cmd))
-      {
-	binding.desc = gettext (binding.desc);
-	break;
-      }
-
   logprintf ("key_events: desc='%s', cmd='%s', keyval=%d, state=%d\n",
 	     binding.desc ? binding.desc : "", binding.cmd, binding.keyval,
 	     binding.state);
@@ -572,7 +660,7 @@ static void xml2kb (xml_node_t *node, int version)
 
 static void load_key_bindings (void)
 {
-  gchar *fname = g_strconcat (g_get_home_dir(), "/.gxine/keybindings", NULL);
+  gchar *fname = get_config_filename (FILE_KEYBINDINGS);
   gchar *kbfile = read_entire_file (fname, NULL);
   int version = 1;
 
@@ -613,20 +701,26 @@ static void load_key_bindings (void)
 	foreach_glist (node, root->child)
 	  if (!strcasecmp (node->name, "KEYBINDING"))
 	    xml2kb (node->child, version);
-      } else {
+      }
+      else
+      {
 	g_printerr (_("key_events: error: %s is not a valid gxine keybindings file\n"),
 		 fname);
 	load_default_kb();
       }
       xml_parser_free_tree (root);
-    } else {
+    }
+    else
+    {
       g_printerr (_("key_events: error: cannot load keybindings file %s (XML parsing failed)\n"),
 	       fname);
       load_default_kb();
     }
 
     free (kbfile);
-  } else {
+  }
+  else
+  {
     if (errno != ENOENT)
       /* the file exists but we can't open it; permissions problem? */
       g_printerr (_("key_events: error: cannot open keybindings file %s\n"),
@@ -637,12 +731,15 @@ static void load_key_bindings (void)
   g_free(fname);
 }
 
-void kb_edit_show (void) {
-
-  if (list_visible) {
+void kb_edit_show (void)
+{
+  if (list_visible)
+  {
     close_edit_window ();
     close_list_window ();
-  } else {
+  }
+  else
+  {
     list_visible = TRUE;
     window_show (kb_bindings_list_dlg, NULL);
     gtk_tree_view_columns_autosize (GTK_TREE_VIEW(tree_view));
@@ -653,13 +750,14 @@ void kb_edit_show (void) {
 static void do_edit_binding (void)
 {
   editkey = *lookup_binding (&catch_key_iter);
-  char *title;
+  editdesc = editkey.desc;
+  editkey.desc = getdesc (&editkey);
 
   gtk_entry_set_text (GTK_ENTRY (kb_binding_desc), editkey.desc);
   gtk_text_buffer_set_text (kb_binding_command_buf, editkey.cmd, -1);
   gtk_entry_set_text (GTK_ENTRY (kb_binding_key), editkey.keyname);
 
-  title = g_strdup_printf (_("Keybinding: %s"), editkey.desc);
+  char *title = g_strdup_printf (_("Keybinding: %s"), editkey.desc);
   gtk_window_set_title (GTK_WINDOW (kb_binding_edit_dlg), title);
   free (title);
 
@@ -670,7 +768,7 @@ static void do_edit_binding (void)
 
 static void kb_new_binding (GtkWidget *widget, gpointer data)
 {
-  
+
   set_key_binding (&default_binding);
   find_key_binding (default_binding.desc, default_binding.cmd, &catch_key_iter);
   do_edit_binding ();
@@ -815,14 +913,19 @@ static gboolean do_edit_key_cb (GtkWidget *widget, GdkEventKey *event,
 	gtk_window_activate_key (GTK_WINDOW (kb_binding_edit_dlg), event))
       return TRUE;
 
-  /* Keys not allowed */
-  if (event->state == 0)
-    switch (event->keyval)
-    {
-    case GDK_Menu: /* menu popup keys */
-    case GDK_F10:
-      return TRUE;
-    }
+  /* Keys not allowed:
+   * Shift-F10 (GTK hardwired menu popup);
+   * GTK menu bar binding (default is F10);
+   * Menu.
+   */
+  if (event->keyval == GDK_F10 &&
+      (event->state & GXINE_MODIFIER_MASK) == GDK_SHIFT_MASK)
+    return TRUE;
+  if (event->keyval == GDK_Menu && !(event->state & GXINE_MODIFIER_MASK))
+    return TRUE;
+  
+  if (is_menu_accel (widget, event))
+    return TRUE;
 
   set_binding_quote (FALSE);
 
@@ -872,7 +975,11 @@ static void apply_edit (void)
 
   key_binding = lookup_binding (&catch_key_iter);
 
-  editkey.desc = gtk_entry_get_text (GTK_ENTRY (kb_binding_desc));
+  const char *desc = gtk_entry_get_text (GTK_ENTRY (kb_binding_desc));
+  /* use the original English version if we have an unmodified translation */
+  editkey.desc = (editkey.std && !strcmp (desc, editkey.desc))
+		 ? editdesc : desc;
+
   gtk_text_buffer_get_bounds (kb_binding_command_buf, &start, &end);
   editkey.cmd = gtk_text_buffer_get_text (kb_binding_command_buf, &start, &end,
 					  FALSE);
@@ -895,7 +1002,7 @@ static JSBool js_xine_event (int type
 #ifdef LOG
 			    , const char *log
 #endif
-			    )
+			   )
 {
   xine_event_t event;
   se_log_fncall (log);
@@ -924,8 +1031,8 @@ JS_XINE_EVENT (input_right, INPUT_RIGHT);
 JS_XINE_EVENT (input_select, INPUT_SELECT);
 
 static JSBool js_input_menu (JSContext *cx, JSObject *obj, uintN argc,
-			     jsval *argv, jsval *rval) {
-
+			     jsval *argv, jsval *rval)
+{
   se_t *se = (se_t *) JS_GetContextPrivate(cx);
   xine_event_t event;
   int32 menu;
@@ -937,7 +1044,8 @@ static JSBool js_input_menu (JSContext *cx, JSObject *obj, uintN argc,
 
   JS_ValueToInt32 (cx, argv[0], &menu);
 
-  if (menu < 1 || menu > 7) {
+  if (menu < 1 || menu > 7)
+  {
     se->print_cb (se->print_cb_data,
 		  _("error: input_menu() expects an int between 1 and 7.\n"));
     return JS_TRUE;
@@ -960,7 +1068,7 @@ JS_XINE_EVENT (input_menu3, INPUT_MENU3);
 JS_XINE_EVENT (input_previous, INPUT_PREVIOUS);
 JS_XINE_EVENT (input_next, INPUT_NEXT);
 
-static JSBool js_keybindings_show (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_keybindings_show (JSContext *cx, JSObject *obj, uintN argc,
 				   jsval *argv, jsval *rval)
 {
   se_log_fncall_checkinit ("keybindings_show");
@@ -968,7 +1076,7 @@ static JSBool js_keybindings_show (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-static JSBool js_keypad_show (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_keypad_show (JSContext *cx, JSObject *obj, uintN argc,
 			      jsval *argv, jsval *rval)
 {
   se_log_fncall_checkinit ("keypad_show");
@@ -1082,6 +1190,7 @@ static const kb_xine_event_map_t xine_vdr = {
     { "BLUE", XINE_EVENT_VDR_BLUE },
     { "CHANNELMINUS", XINE_EVENT_VDR_CHANNELMINUS },
     { "CHANNELPLUS", XINE_EVENT_VDR_CHANNELPLUS },
+    { "CHANNELPREV", XINE_EVENT_VDR_MUTE + 3 /* ..._CHANNELPREVIOUS */ },
     { "CHANNELS", XINE_EVENT_VDR_CHANNELS },
     { "COMMANDS", XINE_EVENT_VDR_COMMANDS },
     { "FASTFWD", XINE_EVENT_VDR_FASTFWD },
@@ -1163,6 +1272,7 @@ JSBool js_event_generic (JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
 
   str = JS_ValueToString (cx, argv[0]);
   evstr = JS_GetStringBytes (str);
+  event.type = 0;
 
   va_start (ap, prefix);
   while ((evlist = va_arg (ap, const kb_xine_event_map_t *)))
@@ -1228,12 +1338,12 @@ static void edit_response_cb (GtkDialog *dbox, int response, gpointer data)
   }
 }
 
-void key_events_init (void) {
-
+void key_events_init (void)
+{
   GtkWidget            *b, *scrolled_window, *hbox;
   GtkCellRenderer      *cell;
   GtkTreeViewColumn    *column;
-   
+
   /*
    * init list store
    */
@@ -1241,7 +1351,6 @@ void key_events_init (void) {
   kb_store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
   kb_model = GTK_TREE_MODEL (kb_store);
 
-  default_binding.desc = gettext (default_binding.desc);
   load_key_bindings ();
 
   /*
@@ -1261,19 +1370,19 @@ void key_events_init (void) {
 
   /* add a nice tree view widget here */
 
-  tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(kb_store));  
+  tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(kb_store));
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree_view), TRUE);
   gtk_tree_view_set_reorderable (GTK_TREE_VIEW (tree_view), TRUE);
 
   cell = gtk_cell_renderer_text_new ();
   column = gtk_tree_view_column_new_with_attributes (_("Action"), cell,
-						     "text", 0, NULL);
+						     "text", COLUMN_DESC, NULL);
   gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN(column), TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view),
 			       GTK_TREE_VIEW_COLUMN (column));
 
   column = gtk_tree_view_column_new_with_attributes (_("Accelerator key"), cell,
-						     "text", 1, NULL);
+						     "text", COLUMN_KEY, NULL);
   gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN(column),
 				   GTK_TREE_VIEW_COLUMN_AUTOSIZE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view),
@@ -1359,7 +1468,7 @@ void key_events_init (void) {
   gtk_entry_set_max_length (GTK_ENTRY(kb_binding_desc), 32);
   add_table_row_items (b, 0, FALSE, _("Description"), kb_binding_desc, NULL);
   gtk_entry_set_activates_default (GTK_ENTRY(kb_binding_desc), TRUE);
-  
+
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -1380,7 +1489,7 @@ void key_events_init (void) {
   add_table_row_items (b, 2, FALSE, _("Accelerator key"),
 		       kb_binding_key, kb_binding_quote, NULL);
 
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (kb_binding_edit_dlg)->vbox ), b);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (kb_binding_edit_dlg)->vbox) , b);
 
   g_object_connect (G_OBJECT (kb_binding_edit_dlg),
 	"signal::delete-event", G_CALLBACK (close_edit_cb), NULL,
