@@ -1,8 +1,8 @@
 /*
  * Copyright (C) 2002-2005 the xine-project
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
@@ -38,11 +38,13 @@
 static GtkWidget *win;
 static GtkTextView *texts[MAX_TEXT];
 static gboolean is_visible = FALSE;
+static gboolean have_datestamps;
 
 static void update_log_window (void)
 {
+  const int count = xine_get_log_section_count (xine);
   int i;
-  for (i = 0; i < xine_get_log_section_count (xine); ++i)
+  for (i = 0; i < count; ++i)
   {
     GtkTextBuffer *buffer = gtk_text_view_get_buffer (texts[i]);
     GtkTextIter start, end;
@@ -57,7 +59,20 @@ static void update_log_window (void)
     {
       int j;
       for (j = 0; log[j]; ++j)
-	gtk_text_buffer_insert (buffer, &end, log[j], -1);
+      {
+	if (have_datestamps)
+	{
+	  const char *colon = strstr (log[j], ": ");
+	  if (colon)
+	    gtk_text_buffer_insert_with_tags_by_name
+	      (buffer, &end, log[j], ++colon - log[j], "datestamp", NULL);
+	  else
+	    colon = log[j];
+	  gtk_text_buffer_insert (buffer, &end, colon, -1);
+	}
+	else
+	  gtk_text_buffer_insert (buffer, &end, log[j], -1);
+      }
     }
   }
 }
@@ -75,7 +90,7 @@ static void response_cb (GtkDialog *dbox, int response, gpointer data)
   }
 }
 
-static JSBool js_log_show (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_log_show (JSContext *cx, JSObject *obj, uintN argc,
 			   jsval *argv, jsval *rval)
 {
   se_log_fncall ("log_show");
@@ -85,6 +100,8 @@ static JSBool js_log_show (JSContext *cx, JSObject *obj, uintN argc,
 
 void log_window_init (void)
 {
+  have_datestamps = xine_check_version (1, 1, 3);
+
   /* window */
 
   win = gtk_dialog_new_with_buttons (_("xine engine log output"), NULL, 0,
@@ -110,6 +127,13 @@ void log_window_init (void)
     gtk_text_view_set_left_margin (texts[i], 2);
     gtk_text_view_set_right_margin (texts[i], 2);
 
+    if (have_datestamps)
+      gtk_text_buffer_create_tag (gtk_text_view_get_buffer (texts[i]),
+				  "datestamp",
+				  "family", "mono",
+				  "weight", PANGO_WEIGHT_BOLD,
+				  NULL);
+
     GtkWidget *scroller = gtk_scrolled_window_new (NULL, NULL);
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroller),
 				    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
