@@ -24,6 +24,7 @@
  */
 
 #include "globals.h"
+#include "version.h"
 
 #include <string.h>
 #include <signal.h>
@@ -95,7 +96,8 @@ gxine_try_remote (int argc, char *argv[], gboolean enqueue, gboolean autoplay)
   if (!enqueue)
     server_client_send ("playlist_clear();\n");
 
-  if (optind<argc) {
+  if (optind < argc)
+  {
     int i;
     for (i = optind; i < argc; ++i)
     {
@@ -123,7 +125,7 @@ static void splash_show (void)
   gchar     *pathname;
   GtkWidget *splash, *fixed, *img, *text;
 
-  gdk_threads_enter();  
+  gdk_threads_enter();
 
   splash = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   GtkWindow *w = GTK_WINDOW (splash);
@@ -133,7 +135,7 @@ static void splash_show (void)
   gtk_window_set_keep_above (w, TRUE);
   gtk_window_stick (w);
 
-  pathname = g_strconcat (pixmapdir, "/splash.png", NULL);
+  pathname = g_build_filename (pixmapdir, "splash.png", NULL);
   img = gtk_image_new_from_file (pathname);
   g_free (pathname);
 
@@ -155,7 +157,7 @@ static void splash_show (void)
   gtk_timeout_add (4000, splash_destroy, splash);
 
   do_pending_events ();
-  gdk_threads_leave();  
+  gdk_threads_leave();
 }
 
 #ifndef GLIB_USES_SYSTEM_MALLOC
@@ -205,6 +207,9 @@ static gboolean post_init_configure (void)
 				&entry))
     vis_set (entry.enum_values[entry.num_value]);
 
+  if (xine_config_lookup_entry (xine, "gui.windowedmode_unblank", &entry))
+    gtk_video_set_windowed_unblank ((GtkVideo *)gtv, entry.num_value);
+
   return FALSE;
 }
 
@@ -237,8 +242,49 @@ static void watchdog_timeout (int sig)
 }
 #endif
 
+/* Our own locking for GDK */
+
+static GStaticRecMutex gdk_lock = G_STATIC_REC_MUTEX_INIT;
+
+static void gxine_gdk_lock (void)
+{
+  g_static_rec_mutex_lock (&gdk_lock);
+}
+
+static void gxine_gdk_unlock (void)
+{
+  g_static_rec_mutex_unlock (&gdk_lock);
+}
+
+/* Faults */
+
+static void fault_handler (int sig)
+{
+  static gboolean recurse = FALSE;
+  if (recurse)
+    abort ();
+
+  signal (sig, fault_handler);
+  recurse = TRUE;
+
+  const char *sigtext = strsignal (sig);
+  fputs (_("gxine has suffered a fatal internal error.\n"
+	   "To get a backtrace, run gxine in a debugger such as gdb.\n"
+	   "Then, when the error occurs:\n"
+	   "  (gdb) thread apply all bt\n"), stderr);
+  display_error_modal (FROM_GXINE, _("Fatal error"), "%s",
+		       sigtext ? : _("Unknown"));
+  exit (2);
+}
+
 /* Main */
 
+static char *
+getdir (const char *env, const char *dir)
+{
+  return getenv (env) ? : g_strconcat (dir, NULL);
+}
+
 int main(int argc, char* argv[])
 {
   gboolean enqueue, autoplay, fullscreen;
@@ -246,6 +292,7 @@ int main(int argc, char* argv[])
   int optstate;
   int show_splash = 2; /* default */
   const char *cmd = NULL;
+  char *geometry = NULL;
 
 #ifndef GLIB_USES_SYSTEM_MALLOC
   GMemVTable vtab = { xmalloc, xrealloc, free, NULL, NULL, NULL };
@@ -260,25 +307,22 @@ int main(int argc, char* argv[])
    * be overwritten by environment variables
    */
 
-  plugindir = getenv("GXINE_PLUGINDIR")	? : g_strconcat(GXINE_PLUGINDIR, NULL);
-  bindir    = getenv("GXINE_BINDIR")	? : g_strconcat(GXINE_BINDIR, NULL);
-  logodir   = getenv("GXINE_LOGODIR")	? : g_strconcat(GXINE_LOGODIR, NULL);
-  pixmapdir = getenv("GXINE_PIXMAPDIR")	? : g_strconcat(GXINE_PIXMAPDIR, NULL);
-  icondir   = getenv("GXINE_ICONDIR")	? : g_strconcat(GXINE_ICONDIR, NULL);
-  miscdir   = getenv("GXINE_MISCDIR")	? : g_strconcat(GXINE_MISCDIR, NULL);
-  confdir   = getenv("GXINE_CONFDIR")	? : g_strconcat(GXINE_CONFDIR, NULL);
+  plugindir = getdir ("GXINE_PLUGINDIR", GXINE_PLUGINDIR);
+  bindir    = getdir ("GXINE_BINDIR", GXINE_BINDIR);
+  logodir   = getdir ("GXINE_LOGODIR", GXINE_LOGODIR);
+  pixmapdir = getdir ("GXINE_PIXMAPDIR", GXINE_PIXMAPDIR);
+  icondir   = getdir ("GXINE_ICONDIR", GXINE_ICONDIR);
+  miscdir   = getdir ("GXINE_MISCDIR", GXINE_MISCDIR);
+  confdir   = getdir ("GXINE_CONFDIR", GXINE_CONFDIR);
 
   /* set up our gtkrc stuff */
   {
     gchar **file = gtk_rc_get_default_files ();
-    gchar **copy;
     const gchar *home = g_get_home_dir ();
     guint homelen = strlen (home);
-    guint length = 0, i;
-    while (file[length])
-      ++length;
+    guint i, length = g_strv_length (file);
     /* deep-copy the gtkrc filenames, allocating space for two more */
-    copy = calloc (length + 3, sizeof (gchar *));
+    gchar **copy = calloc (length + 3, sizeof (gchar *));
     for (i = 0; file[i]; ++i)
       copy[i] = strdup (file[i]);
     /* find the index of the first *user* gtkrc filename */
@@ -287,15 +331,13 @@ int main(int argc, char* argv[])
 	break;
     /* insert our system gtkrc filename before it (strdup for convenience) */
     memmove (copy + i + 1, copy + i, (length - i) * sizeof (gchar *));
-    copy[i] = g_strconcat (confdir, "/gtkrc", NULL);
+    copy[i] = g_build_filename (confdir, FILE_GTKRC, NULL);
     /* now append our user gtkrc, terminate the list and tell GTK+ about it */
-    copy[length + 1] = g_strconcat (home, "/.gxine/gtkrc", NULL);
+    copy[length + 1] = get_config_filename (FILE_GTKRC);
     copy[length + 2] = NULL;
     gtk_rc_set_default_files (copy);
     /* free the list */
-    for (i = 0; copy[i]; ++i)
-      free (copy[i]);
-    free (copy);
+    g_strfreev (copy);
   }
 
   /*
@@ -317,9 +359,11 @@ int main(int argc, char* argv[])
 
   g_thread_init (NULL);
   console_output_init ();
+  gdk_threads_set_lock_functions (gxine_gdk_lock, gxine_gdk_unlock);
   gdk_threads_init ();
-  gtk_init(&argc, &argv);
-  gtk_window_set_auto_startup_notification (FALSE);
+
+  if (!gtk_parse_args (&argc, &argv))
+    return 2;
 
   /*
    * parse command line arguments
@@ -347,6 +391,7 @@ int main(int argc, char* argv[])
       { "splash", no_argument, NULL, 's' },
       { "no-splash", no_argument, NULL, 'S' },
       { "command", required_argument, NULL, 'c' },
+      { "geometry", required_argument, NULL, 2 },
       { NULL }
     };
     int index = 0;
@@ -357,13 +402,17 @@ int main(int argc, char* argv[])
     if (opt == -1)
       break;
 
-    switch (opt) {
+    switch (opt)
+    {
     case 'h':
       optstate |= 1;
       break;
     case 1:
       optstate |= 4;
       break;
+    case 2:
+      geometry = optarg;
+      break;
     case 'v':
       verbosity++;
       break;
@@ -416,10 +465,10 @@ options:\n\
   -e, --enqueue		don't clear the playlist, don't play the new items\n\
   -f, --full-screen	start in full-screen mode\n\
   -v, --verbose		be more verbose\n\
-\n"), VERSION, VENDOR_PKG_VERSION, argv[0]);
+\n"), VERSION, VENDOR_PKG_VERSION CSET_ID, argv[0]);
   else if (optstate & 4)
-    g_print ("gxine %s (%s)\n%s\n%s", VERSION, VENDOR_PKG_VERSION,
-	    _("© 2002-2006 the xine project team"), _("\
+    g_print ("gxine %s (%s)\n%s\n%s", VERSION, VENDOR_PKG_VERSION CSET_ID,
+	     get_copyright_notice (), _("\
 This is free software; see the source for copying conditions.  There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE,\n\
 to the extent permitted by law.\n"));
@@ -433,6 +482,13 @@ to the extent permitted by law.\n"));
   if (optstate)
     return 0;
 
+  /* init gtk windowing */
+
+  gtk_init(&argc, &argv);
+  gtk_window_set_auto_startup_notification (FALSE);
+
+  signal (SIGSEGV, fault_handler);
+
   /*
    * find out if gxine is already running, if so
    * just pass on the files to play
@@ -453,7 +509,7 @@ to the extent permitted by law.\n"));
    * make sure ~/.gxine exists
    */
   {
-    char *fname = g_strconcat (g_get_home_dir(), "/.gxine", NULL);
+    char *fname = get_config_filename (NULL);
     int r = ensure_path_exists (fname, 0700);
     if (r)
       display_warning
@@ -505,10 +561,17 @@ to the extent permitted by law.\n"));
   ui_init ();
 
   gtk_window_set_default_icon_name (GXINE_LOGO);
+  
+
+  {
+    char *fname = get_config_filename (FILE_ACCELS);
+    gtk_accel_map_load (fname);
+    free (fname);
+  }
 
   do_pending_events ();
   gtk_window_set_auto_startup_notification (TRUE);
-  noskin_main_init ();
+  noskin_main_init (geometry);
 
   do_pending_events ();
 
@@ -549,21 +612,19 @@ to the extent permitted by law.\n"));
    * argument parsing
    */
 
-  if (optind<argc) {
+  if (optind < argc)
+  {
     int i;
     static int first = -1;
 
     if (!enqueue)
       playlist_clear();
 
-    for (i=optind; i<argc; i++) {
-
-      if (first<0) {
+    for (i = optind; i < argc; ++i)
+      if (first < 0)
 	first = playlist_add_mrl (argv[i], -1);
-      } else {
+      else
 	playlist_add_mrl (argv[i], -1);
-      }
-    }
 
     if (first < 0)
       gtk_init_add ((GtkFunction) playlist_logo, NULL);
