@@ -1,4 +1,4 @@
-/* 
+/*
  * Copyright (C) 2003-2005 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
@@ -13,9 +13,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: menu.h,v 1.4 2005/01/03 22:06:04 dsalt Exp $
+ * $Id: menu.h,v 1.8 2006-02-21 19:06:12 dsalt Exp $
  *
  * main/context menu creation / handling
  */
@@ -23,13 +23,17 @@
 #ifndef HAVE_MENU_H
 #define HAVE_MENU_H
 
-#include <gtk/gtk.h>
+#include <gtk/gtkwidget.h>
 
-#define num_menu_items(tree) (sizeof ((tree)) / sizeof ((tree)[0]))
-
-GtkWidget *create_menu_tree (GtkItemFactory **, GtkWidget *window,
-			     GtkAccelGroup **, GtkItemFactoryEntry *,
-			     gint items);
 void create_menus (GtkWidget *window);
+void menu_add_items (GtkUIManager *, const char *const prefix,
+		     const char *const label, guint merge, size_t count);
+const char *menu_get_tree_name (unsigned int);
+
+/* signal callback & support for dynamic scale changes */
+void scale_changed_cb (GtkWidget *, gdouble, gpointer);
+GtkAction *find_video_size_action (gdouble);
+
+extern GtkUIManager *ui;
 
 #endif
