@@ -1,4 +1,4 @@
-/* 
+/*
  * Copyright (C) 2003-2005 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,8 +25,7 @@
 
 #include <glib.h>
 
-void noskin_main_init (void) ;
-
+void noskin_main_init (const gchar *geometry);
 void noskin_main_close (void);
 
 #endif
