@@ -18,7 +18,7 @@
  *
  * $Id: player.c,v 1.86 2006/04/08 21:34:50 dsalt Exp $
  *
- * player object 
+ * player object
  */
 
 #include "globals.h"
@@ -61,7 +61,7 @@ static int             goto_pos, goto_time;
 static pthread_t       play_thread;
 static char           *cur_mrl = NULL, *cur_title = NULL;
 
-static __attribute__ ((noreturn)) void *play_exec (void *queue_gen)
+static __attribute__ ((noreturn)) void *play_exec (void *data)
 {
   xine_cfg_entry_t entry;
   char *hash, *mrl = NULL;
@@ -139,7 +139,8 @@ static __attribute__ ((noreturn)) void *play_exec (void *queue_gen)
   {
     int error;
     gdk_threads_enter ();
-    switch (error = xine_get_error (stream)) {
+    switch (error = xine_get_error (stream))
+    {
     case XINE_ERROR_NO_INPUT_PLUGIN:
       display_error (FROM_XINE, _("The xine engine failed to start."),
 		     _("No input plugin was found.\n"
@@ -148,23 +149,38 @@ static __attribute__ ((noreturn)) void *play_exec (void *queue_gen)
       break;
 
     case XINE_ERROR_NO_DEMUX_PLUGIN:
-      display_error (FROM_XINE, _("The xine engine failed to start."),
+      display_error (FROM_XINE,
+		     playlist_showing_logo ()
+		       ? _("Whoops. You seem to have a broken xine-lib.")
+		       : _("The xine engine failed to start."),
 		     _("No demuxer found - stream format not recognised."));
       break;
 
+    case XINE_ERROR_DEMUX_FAILED:
+      display_error (FROM_XINE, _("The xine engine failed to start."),
+		     _("Demuxing failed.\n"
+		       "Maybe the stream is corrupt or of an unexpected type."));
+      break;
+
     case XINE_ERROR_MALFORMED_MRL:
       display_error (FROM_XINE, _("The xine engine failed to start."),
 		     _("The MRL is malformed."));
       break;
 
+    case XINE_ERROR_INPUT_FAILED:
+      display_error (FROM_XINE, _("The xine engine failed to start."),
+		     _("The stream could not be opened."));
+      break;
+
+
     default:
       display_error (FROM_XINE, _("The xine engine failed to start."),
 		     _("Unknown error (code %d)"), error);
     }
 
-    g_idle_add ((GSourceFunc)playlist_logo, NULL);
-    gdk_threads_leave ();
-    goto done;
+    if (!playlist_showing_logo ())
+      g_idle_add ((GSourceFunc)playlist_logo, NULL);
+    goto done; /* will call gdk_threads_leave */
   }
 
   gdk_threads_enter ();
@@ -214,10 +230,16 @@ void player_launch (const char *title, const char *mrl, int pos, int pos_time)
     abort();
   }
 
+  if (!playlist_showing_logo ())
+    pthread_detach (play_thread);
+
   pthread_attr_destroy (&attr);
-  pthread_detach (play_thread);
 }
 
+void player_wait (void)
+{
+  pthread_join (play_thread, NULL);
+}
 
 static void set_mute (int mute)
 {
@@ -295,7 +317,7 @@ static void set_subtitles (int sub)
     GSList *menuitem = action_items.subtitles;
     int count = g_slist_length (menuitem) - 3;
     if (count >= sub)
-    {    
+    {
       while (menuitem && --count > sub)
 	menuitem = menuitem->next;
       if (menuitem)
@@ -348,6 +370,12 @@ static void get_ao_postproc (se_prop_t *prop, se_prop_read_t *value)
   value->i = gtk_video_get_use_post_plugins_audio ((GtkVideo *)gtv);
 }
 
+ACTION_SET(wm_unblank)
+static void get_wm_unblank (se_prop_t *prop, se_prop_read_t *value)
+{
+  value->i = gtk_video_get_windowed_unblank ((GtkVideo *)gtv);
+}
+
 static void update_postproc_chain (const char *pref, const char *value)
 {
   xine_cfg_entry_t entry;
@@ -390,7 +418,7 @@ static int listen_postproc_chain (void *pref, se_t *se, se_o_t *obj,
   return 0;
 }
 
-static JSBool get_time (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool get_time (JSContext *cx, JSObject *obj, uintN argc,
 		     jsval *argv, jsval *rval)
 {
   int   pos, pos_time, len;
@@ -402,7 +430,7 @@ static JSBool get_time (JSContext *cx, JSObject *obj, uintN argc,
 
 #ifdef WITH_DEPRECATED
 
-static JSBool js_get_speed (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_get_speed (JSContext *cx, JSObject *obj, uintN argc,
 			    jsval *argv, jsval *rval)
 {
   se_prop_read_t speed;
@@ -412,22 +440,18 @@ static JSBool js_get_speed (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-static JSBool js_get_volume (JSContext *cx, JSObject *obj, uintN argc, 
-			    jsval *argv, jsval *rval) {
-
+static JSBool js_get_volume (JSContext *cx, JSObject *obj, uintN argc,
+			    jsval *argv, jsval *rval)
+{
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
   int   volume;
-
   se_log_fncall_deprecated ("get_volume");
-
   volume = xine_get_param (stream, XINE_PARAM_AUDIO_VOLUME);
-
   *rval = INT_TO_JSVAL (volume);
-
   return JS_TRUE;
 }
 
-static JSBool js_get_mute (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_get_mute (JSContext *cx, JSObject *obj, uintN argc,
 			   jsval *argv, jsval *rval)
 {
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
@@ -438,24 +462,20 @@ static JSBool js_get_mute (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-static JSBool js_get_zoom (JSContext *cx, JSObject *obj, uintN argc, 
-			   jsval *argv, jsval *rval) {
-
+static JSBool js_get_zoom (JSContext *cx, JSObject *obj, uintN argc,
+			   jsval *argv, jsval *rval)
+{
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
   int   zoom;
-
   se_log_fncall_deprecated ("get_zoom");
-
   zoom = xine_get_param (stream, XINE_PARAM_VO_ZOOM_X);
-
   *rval = INT_TO_JSVAL (zoom);
-
   return JS_TRUE;
 }
 
 #endif /* WITH_DEPRECATED */
 
-static JSBool controls_pause (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool controls_pause (JSContext *cx, JSObject *obj, uintN argc,
 			      jsval *argv, jsval *rval)
 {
   int32 pause_state=-1;
@@ -463,12 +483,14 @@ static JSBool controls_pause (JSContext *cx, JSObject *obj, uintN argc,
   se_log_fncall_checkinit ("pause");
   se_argc_check_max (1, "pause");
 
-  if (argc==1) {
+  if (argc == 1)
+  {
     se_arg_is_int_or_bool (0, "pause");
     JS_ValueToInt32 (cx, argv[0], &pause_state);
   }
 
-  switch (pause_state) {
+  switch (pause_state)
+  {
   case 0: /* pause off */
     set_speed (XINE_SPEED_NORMAL);
     break;
@@ -485,19 +507,34 @@ static JSBool controls_pause (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-static JSBool controls_stop (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool controls_stop (JSContext *cx, JSObject *obj, uintN argc,
 			     jsval *argv, jsval *rval)
 {
   se_log_fncall_checkinit ("stop");
   xine_stop (stream);
-  playlist_logo ();
+  playlist_logo (NULL);
   ui_set_control_adjustment (Control_SEEKER, 0);
   return JS_TRUE;
 }
 
+static JSBool controls_eject (JSContext *cx, JSObject *obj, uintN argc,
+			      jsval *argv, jsval *rval)
+{
+  se_log_fncall_checkinit ("eject");
+  if (xine_get_status (stream) == XINE_STATUS_PLAY)
+    xine_stop (stream);
+  gdk_threads_leave (); /* need to do this to allow play_exec() to proceed */
+  playlist_logo (cx); /* doesn't matter so long as it's not NULL */
+  gdk_threads_enter ();
+  ui_set_control_adjustment (Control_SEEKER, 0);
+  int ret = xine_eject (stream);
+  *rval = BOOLEAN_TO_JSVAL (ret);
+  return JS_TRUE;
+}
+
 #ifdef WITH_DEPRECATED
 
-static JSBool js_set_speed (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_set_speed (JSContext *cx, JSObject *obj, uintN argc,
 			    jsval *argv, jsval *rval)
 {
   int32 speed;
@@ -516,7 +553,7 @@ static JSBool js_set_speed (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-static JSBool js_set_volume (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_set_volume (JSContext *cx, JSObject *obj, uintN argc,
 			    jsval *argv, jsval *rval)
 {
   int32 volume;
@@ -535,7 +572,7 @@ static JSBool js_set_volume (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-static JSBool js_set_mute (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_set_mute (JSContext *cx, JSObject *obj, uintN argc,
 			   jsval *argv, jsval *rval)
 {
   JSBool mute;
@@ -556,7 +593,7 @@ static JSBool js_set_mute (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-static JSBool js_set_zoom (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_set_zoom (JSContext *cx, JSObject *obj, uintN argc,
 			   jsval *argv, jsval *rval)
 {
   int32 zoom;
@@ -598,7 +635,7 @@ static gboolean set_video_size_cb (gpointer unused)
   return FALSE;
 }
 
-static JSBool js_set_video_size (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_set_video_size (JSContext *cx, JSObject *obj, uintN argc,
 			   jsval *argv, jsval *rval)
 {
   se_log_fncall ("set_video_size");
@@ -623,37 +660,41 @@ static JSBool js_set_video_size (JSContext *cx, JSObject *obj, uintN argc,
 
 #ifdef WITH_DEPRECATED
 
-static JSBool js_set_fullscreen (JSContext *cx, JSObject *obj, uintN argc, 
-				 jsval *argv, jsval *rval) {
+static JSBool js_set_fullscreen (JSContext *cx, JSObject *obj, uintN argc,
+				 jsval *argv, jsval *rval)
+{
   int32 fs;
 
   se_log_fncall_deprecated ("set_fullscreen");
   se_argc_check_max (1, "set_fullscreen");
 
-  if (argc==1) {
+  if (argc == 1)
+  {
     se_arg_is_int_or_bool (0, "set_fullscreen");
     JS_ValueToInt32 (cx, argv[0], &fs);
-  } else {
-    fs = !gtk_video_is_fullscreen ((GtkVideo *)gtv);
   }
+  else
+    fs = !gtk_video_is_fullscreen ((GtkVideo *)gtv);
 
   set_fullscreen (fs);
   return JS_TRUE;
 }
 
-static JSBool js_set_deinterlace (JSContext *cx, JSObject *obj, uintN argc, 
-				 jsval *argv, jsval *rval) {
+static JSBool js_set_deinterlace (JSContext *cx, JSObject *obj, uintN argc,
+				 jsval *argv, jsval *rval)
+{
   int32 di;
 
   se_log_fncall_deprecated ("set_deinterlace");
   se_argc_check_max (1, "set_deinterlace");
 
-  if (argc==1) {
+  if (argc == 1)
+  {
     se_arg_is_int_or_bool (0, "set_deinterlace");
     JS_ValueToInt32 (cx, argv[0], &di);
-  } else {
-    di = !gtk_video_get_use_post_plugins_deinterlace ((GtkVideo *)gtv);
   }
+  else
+    di = !gtk_video_get_use_post_plugins_deinterlace ((GtkVideo *)gtv);
 
   set_deinterlace (di);
   return JS_TRUE;
@@ -718,18 +759,20 @@ static JSBool js_set_postproc_audio (JSContext *cx, JSObject *obj,
 }
 
 static JSBool js_set_auto_resize (JSContext *cx, JSObject *obj, uintN argc,
-				 jsval *argv, jsval *rval) {
+				 jsval *argv, jsval *rval)
+{
   int32 ar;
 
   se_log_fncall_deprecated ("set_auto_resize");
   se_argc_check_max (1, "set_auto_resize");
 
-  if (argc==1) {
+  if (argc == 1)
+  {
     se_arg_is_int_or_bool (0, "set_auto_resize");
     JS_ValueToInt32 (cx, argv[0], &ar);
-  } else {
-    ar = !gtk_video_get_auto_resize ((GtkVideo *)gtv);
   }
+  else
+    ar = !gtk_video_get_auto_resize ((GtkVideo *)gtv);
 
   set_auto_resize (ar);
   return JS_TRUE;
@@ -743,18 +786,19 @@ static JSBool js_set_auto_rescale (JSContext *cx, JSObject *obj, uintN argc,
   se_log_fncall_deprecated ("set_auto_rescale");
   se_argc_check_max (1, "set_auto_rescale");
 
-  if (argc==1) {
+  if (argc == 1)
+  {
     se_arg_is_int_or_bool (0, "set_auto_rescale");
     JS_ValueToInt32 (cx, argv[0], &ar);
-  } else {
-    ar = !gtk_video_get_auto_rescale ((GtkVideo *)gtv);
   }
+  else
+    ar = !gtk_video_get_auto_rescale ((GtkVideo *)gtv);
 
   set_auto_rescale (ar);
   return JS_TRUE;
 }
 
-static JSBool js_set_aspect (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_set_aspect (JSContext *cx, JSObject *obj, uintN argc,
 			     jsval *argv, jsval *rval)
 {
   int32 aspect;
@@ -762,10 +806,12 @@ static JSBool js_set_aspect (JSContext *cx, JSObject *obj, uintN argc,
   se_log_fncall_deprecated ("set_aspect");
   se_argc_check_max (1, "set_aspect");
 
-  if (argc==1) {
+  if (argc==1)
+  {
     se_arg_is_int (0, "set_aspect");
     JS_ValueToInt32 (cx, argv[0], &aspect);
-  } else
+  }
+  else
     aspect = xine_get_param (stream, XINE_PARAM_VO_ASPECT_RATIO) + 1;
 
   set_aspect (aspect);
@@ -774,7 +820,7 @@ static JSBool js_set_aspect (JSContext *cx, JSObject *obj, uintN argc,
 
 #endif /* WITH_DEPRECATED */
 
-static JSBool js_snapshot (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_snapshot (JSContext *cx, JSObject *obj, uintN argc,
 			   jsval *argv, jsval *rval)
 {
   JSString *str;
@@ -803,75 +849,8 @@ static JSBool js_snapshot (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-/**************************************************************
- *
- * player.controls class & object
- *
- * adapted from http://users.skynet.be/saw/SpiderMonkey.htm
- *
- **************************************************************/
 
-/*
- * controls object properties
- */
-
-#if 0
-int controls_prop_cb (void *user_data, char *property, int *num) {
-  /* FIXME */
-  return 1;
-}
-
-#define CONTROLS_PROP_AUDIO_LANGUAGE_COUNT          0
-#define CONTROLS_PROP_CURRENT_AUDIO_LANGUAGE        1
-#define CONTROLS_PROP_CURRENT_AUDIO_LANGUAGE_INDEX  2
-#define CONTROLS_PROP_CURRENT_ITEM                  3
-#define CONTROLS_PROP_CURRENT_MARKER                4
-#define CONTROLS_PROP_CURRENT_POSITION              5
-#define CONTROLS_PROP_CURRENT_POSITION_STRING       6
-#define CONTROLS_PROP_CURRENT_POSITION_TIMECODE     7
-#define CONTROLS_PROP_IS_AVAILABLE                  8
-
-JSBool controls_JSGetProperty (JSContext *cx, JSObject *obj, jsval id, jsval *vp) {
-
-  se_log_fncall ("get_property");
-
-  if (JSVAL_IS_STRING (id)) {
-    JSString *str;
-    char     *prop;
-
-    se_log ("id is a string\n");
-
-    str = JS_ValueToString (cx, id);
-    prop = JS_GetStringBytes (str);
-
-    se_log ("id = '%s'\n", JS_GetStringBytes (str));
-
-    if (!strcmp (prop, "audioLanguageCount")) {
-      *vp = INT_TO_JSVAL (1);
-    } else if (!strcmp (prop, "currentPosition")) {
-      *vp = INT_TO_JSVAL (0);
-    }
-  }
-#if 0
-  if (JSVAL_IS_INT(id)) 
-    {
-      Customer *priv = (Customer *) JS_GetPrivate(cx, obj);
-      switch(JSVAL_TO_INT(id))
-	{
-	case name_prop:
-	  
-	  break;
-	case age_prop:
-	  *vp = INT_TO_JSVAL(priv->getCustomer()->GetAge());
-	  break;
-	}
-    }
-#endif
-  return JS_TRUE;
-}
-#endif
-  
-static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
 			     jsval *argv, jsval *rval)
 {
   se_log_fncall_checkinit ("play");
@@ -880,7 +859,8 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
   char *mrl = NULL;
   int32 pos = -1, pos_time = -1;
 
-  switch (argc) {
+  switch (argc)
+  {
   case 0:
     break;
 
@@ -896,7 +876,7 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
     }
     break;
 
-  case 2: 
+  case 2:
     {
       se_arg_is_int (0, "play");
       se_arg_is_int (1, "play");
@@ -904,7 +884,7 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
       JS_ValueToInt32 (cx, argv[0], &pos);
       JS_ValueToInt32 (cx, argv[1], &pos_time);
       mrl = NULL;
-      se_log ("playing from %d, %d\n", pos, pos_time); 
+      se_log ("playing from %d, %d\n", pos, pos_time);
     }
     break;
 
@@ -953,7 +933,7 @@ static JSBool controls_play (JSContext *cx, JSObject *obj, uintN argc,
       play_item_dispose (item);
     } else
       logprintf (_("script_engine: error: no valid play item available\n"));
-  } 
+  }
 
   if (xine_get_param (stream, XINE_PARAM_SPEED) != XINE_SPEED_NORMAL)
     set_speed (XINE_SPEED_NORMAL);
@@ -970,9 +950,8 @@ static JSBool controls_play_pause (JSContext *cx, JSObject *obj, uintN argc,
   se_log_fncall_checkinit ("play_pause");
   se_argc_check_max (1, "play_pause");
 
-  if (argc==1) {
+  if (argc == 1)
     se_arg_is_int_or_bool (0, "play_pause");
-  }
 
   switch (xine_get_status (stream))
   {
@@ -1019,45 +998,34 @@ const char *player_get_cur_mrl (void)
 }
 
 
-static xine_audio_port_t *load_audio_out_driver (void) {
- 
+static xine_audio_port_t *load_audio_out_driver (void)
+{
   xine_audio_port_t      *audio_port;
- 
-  if (!audio_driver_id) {
- 
-    char **driver_ids = (char **) xine_list_audio_output_plugins (xine) ;
-    char **choices;
-    int i;
- 
-    choices = malloc (sizeof (char *) * 100);
-    choices[0] = "auto"; i=0;
-    while (driver_ids[i]) {
-      choices[i+1] = strdup(driver_ids[i]);
-      i++;
-    }
-    choices[i+1]=0;
 
+  if (!audio_driver_id)
+  {
+    char **choices = get_driver_ids (xine_list_audio_output_plugins (xine));
     /* try to init audio with stored information */
-    i = xine_config_register_enum (xine,
+    int i = xine_config_register_enum (xine,
                                    "audio.driver", 0,
                                    choices,
                                    _("audio driver to use"),
-                                   NULL, 10, NULL, CONFIG_DATA_NONE);
+                                   NULL, 10, gtk_true, NULL);
     audio_driver_id = choices[i];
   }
- 
+
   if (!strcmp (audio_driver_id, "null"))
     return NULL;
- 
-  if (strcmp (audio_driver_id, "auto")) {
- 
+
+  if (strcmp (audio_driver_id, "auto"))
+  {
     audio_port = xine_open_audio_driver (xine, audio_driver_id, NULL);
     if (audio_port)
       return audio_port;
     else
       g_printerr (_("audio driver %s failed\n"), audio_driver_id);
   }
- 
+
   /* autoprobe */
   return xine_open_audio_driver (xine, NULL, NULL);
 }
@@ -1069,7 +1037,7 @@ void player_init (void)
 /*
   se_o_t            *controls_obj;
  */
-  
+
   cur_mrl = NULL;
 
   /* load drivers, create xine stream */
@@ -1080,7 +1048,7 @@ void player_init (void)
   stream = xine_stream_new (xine, audio_port, video_port);
 
   /*
-   * set up script engine objects and functions 
+   * set up script engine objects and functions
    */
 
 /*
@@ -1111,6 +1079,7 @@ void player_init (void)
       { "vo_deinterlace", N_("v=bool, toggle(): enable deinterlacing"), 0, get_deinterlace, set_deinterlace },
       { "ao_postproc", N_("v=bool, toggle(): audio post-processing; chain=string: plugin chain"), 0, get_ao_postproc, set_ao_postproc },
       { "vo_postproc", N_("v=bool, toggle(): video post-processing; chain=string: plugin chain"), 0, get_vo_postproc, set_vo_postproc },
+      { "wm_unblank", N_("v=bool, toggle(): unblanking in windowed mode"), 0, get_wm_unblank, set_wm_unblank },
       { NULL }
     };
     static const ui_property_t zoom_subprops[] = {
@@ -1156,6 +1125,8 @@ void player_init (void)
   	SE_GROUP_ENGINE, N_("[bool]"), NULL },
       { "stop", controls_stop, 0, 0,
 	SE_GROUP_ENGINE, NULL, NULL },
+      { "eject", controls_eject, 0, 0,
+	SE_GROUP_ENGINE, NULL, NULL },
       { "play_pause", controls_play_pause, 0, 0,
 	SE_GROUP_ENGINE, N_("[bool]"), NULL },
       { "snapshot", js_snapshot, 0, 0,
@@ -1219,11 +1190,11 @@ void player_init (void)
        _("Double the size of small video streams"),
        _("This affects video streams whose dimensions are at most ⅓ "
 	 "of those of the display."),
-       0, NULL, NULL);
+       0, gtk_true, NULL);
     xine_config_register_bool
       (xine, "gui.subtitle_autoload", 1,
        _("Subtitle autoloading"),
        _("Automatically load subtitles if they exist."),
-       0, NULL, NULL);
+       0, gtk_true, NULL);
   }
 }
