@@ -1,8 +1,8 @@
 /*
  * Copyright (C) 2002-2004 the xine-project
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
@@ -33,7 +33,7 @@ void playlist_clear (void);
 int playlist_flush (play_item_type);
 
 int  playlist_add     (play_item_t *play_item, gint ins_pos /* -1 => append */);
-int  playlist_add_mrl (const char *mrl, gint ins_pos /* -1 => append */) ;
+int  playlist_add_mrl (const char *mrl, gint ins_pos /* -1 => append */);
 void playlist_add_list (GSList *, gboolean play); /* also frees the list */
 
 void playlist_check_set_title (void);
@@ -41,7 +41,7 @@ void playlist_check_set_title (void);
 void playlist_play      (int list_pos);
 void playlist_play_from (int list_pos, int pos, int pos_time);
 
-void playlist_logo (void);
+void playlist_logo (gpointer wait_if_non_null);
 
 int playlist_showing_logo (void);
 
