@@ -62,6 +62,7 @@ struct gxine_post_s {
   xine_post_in_t *info;
   GtkWidget *box, *enabled, *pref, *up;	/* in parent dbox */
   GtkWidget *dbox, **conf_w;	/* this plugin's dbox */
+  guint pref_toggled;		/* signal id */
   char *params;			/* raw config */
   void (*reconfigure)(struct gxine_post_s *);
 };
@@ -372,6 +373,17 @@ post_config_post_dispose_dbox (gxine_post_t *info)
   info->conf_w = NULL;
 }
 
+static gboolean
+post_config_post_window (GtkButton *widget, gxine_post_t *info);
+
+static void
+toggle_pref_button (const gxine_post_t *info, gboolean state)
+{
+  g_signal_handler_block (G_OBJECT (info->pref), info->pref_toggled);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (info->pref), state);
+  g_signal_handler_unblock (G_OBJECT (info->pref), info->pref_toggled);
+}
+
 static void
 post_config_post_response (GtkDialog *dbox, gint response, gxine_post_t *info)
 {
@@ -388,8 +400,7 @@ post_config_post_response (GtkDialog *dbox, gint response, gxine_post_t *info)
     /* fall through */
   default:
     post_config_post_dispose_dbox (info);
-    if (info->pref)
-      gtk_widget_set_sensitive (info->pref, TRUE);
+    toggle_pref_button (info, FALSE);
     break;
   }
 }
@@ -467,7 +478,7 @@ post_config_post_reconfigure (gxine_post_t *info)
   info->parameters = params;
 }
 
-static JSBool
+static gboolean
 post_config_post_window (GtkButton *widget, gxine_post_t *info)
 {
   const xine_post_api_t *api;
@@ -479,6 +490,7 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
 
   if (info->dbox)
   {
+    toggle_pref_button (info, TRUE);
     gtk_window_present (GTK_WINDOW(info->dbox));
     return JS_FALSE;
   }
@@ -592,6 +604,9 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
     gtk_table_attach (GTK_TABLE(table), widget,
 		      1, 2, param_count, param_count + 1, 7, 7, 2, 2);
 
+    if (param->description)
+      gtk_tooltips_set_tip (tips, widget, param->description, NULL);
+
     widget = gtk_label_new (param->name);
     gtk_misc_set_alignment (GTK_MISC (widget), 1, 0.5);
     gtk_table_attach (GTK_TABLE(table), widget,
@@ -620,19 +635,37 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
 
       gtk_container_add (GTK_CONTAINER(window), help);
       gtk_text_view_set_editable ((GtkTextView *)help, FALSE);
-      gtk_text_view_set_wrap_mode ((GtkTextView *)help, 3);
-	/* earlier GTK+2.2 apparently doesn't define GTK_WRAP_WORD_CHAR */
 
       /* use the normal fg/bg colours, rather than the text display colours */
       g_signal_connect (G_OBJECT(help), "realize",
 			G_CALLBACK(help_realise_cb), NULL);
 
-      /* use normal font or, if there's at least one tab, monospaced;
+      /* use normal font or, if there are any tabs or no long lines,
+       * a monospaced font (it's better to err in this direction);
        * either way, reduce the size a little
        */
-      font = strchr (helptext, '\t')
-             ? pango_font_description_from_string ("mono")
-	     : pango_font_description_copy_static (help->style->font_desc);
+      gboolean mono = TRUE, wrap = FALSE;
+      char *lf1 = helptext - 1;
+      while (lf1)
+      {
+	char *lf2 = strchr (lf1 + 1, '\n');
+	if ((lf2 ? lf2 : lf1 + strlen (lf1)) - lf1 > 82)
+	{
+	  mono = FALSE;
+	  wrap = TRUE;
+	  break;
+	}
+	lf1 = lf2;
+      }
+      if (strchr (helptext, '\t'))
+	mono = TRUE;
+
+      gtk_text_view_set_wrap_mode ((GtkTextView *)help,
+				   wrap ? GTK_WRAP_WORD_CHAR : GTK_WRAP_NONE);
+
+      font = mono ? pango_font_description_from_string ("mono")
+		  : pango_font_description_copy_static (help->style->font_desc);
+
       p = pango_font_description_get_size (help->style->font_desc);
       pango_font_description_set_size (font, p * 5 / 6);
       gtk_widget_modify_font (help, font);
@@ -642,11 +675,12 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
 
       gtk_text_buffer_set_text (buffer, helptext, -1);
       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(window),
-				      GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-      gtk_widget_set_size_request (window, 1, 16);
+				      wrap ? GTK_POLICY_NEVER : GTK_POLICY_AUTOMATIC,
+				      GTK_POLICY_AUTOMATIC);
+
+      gtk_widget_set_size_request (window, 1, -1);
       gtk_box_pack_end (GTK_BOX(GTK_DIALOG(info->dbox)->vbox), window,
 			TRUE, TRUE, 2);
-
     }
     free (helptext);
 
@@ -665,8 +699,6 @@ post_config_post_window (GtkButton *widget, gxine_post_t *info)
 				   GDK_HINT_RESIZE_INC);
   }
 
-  if (info->pref)
-    gtk_widget_set_sensitive (info->pref, FALSE);
   return JS_TRUE;
 }
 
@@ -879,12 +911,14 @@ post_config_post_add_plugin (gxine_chain_t *chain, xine_post_t *post,
   g_signal_connect (G_OBJECT(w), "clicked",
 		    G_CALLBACK(post_config_post_chain_delete_plugin), info);
 
-  info->pref = w = gtk_button_new_from_stock (GTK_STOCK_PROPERTIES);
+  info->pref = w = gtk_toggle_button_new_with_label (GTK_STOCK_PROPERTIES);
+  g_object_set (G_OBJECT(w), "use-stock", TRUE, NULL);
   gtk_box_pack_end (GTK_BOX(info->box), w, FALSE, TRUE, 0);
   if (!xine_post_input (post, "parameters"))
     gtk_widget_set_sensitive (w, FALSE);
-  g_signal_connect (G_OBJECT(w), "clicked",
-		    G_CALLBACK(post_config_post_window), info);
+  info->pref_toggled = g_signal_connect (G_OBJECT(w), "toggled",
+					 G_CALLBACK(post_config_post_window),
+					 info);
 
   gtk_box_pack_start (GTK_BOX(GTK_DIALOG(chain->dbox)->vbox), info->box,
 		      FALSE, TRUE, 2);
@@ -1005,7 +1039,7 @@ post_config_post_chain_window (gxine_chain_t *chain)
   plugin_list = xine_list_post_plugins_typed (xine, post_type[chain->type]);
   if (!plugin_list || !plugin_list[0])
   {
-    title = g_markup_printf_escaped (_("Plugin chain %s"),
+    title = g_markup_printf_escaped (_("Plugin chain: %s"),
 				     gettext (chain_title[chain->type]));
     display_info (FROM_XINE, title,
 		  _("No available plugins - nothing to configure"));
@@ -1020,7 +1054,7 @@ post_config_post_chain_window (gxine_chain_t *chain)
 
   logprintf ("post_config: start\n");
 
-  title = g_strdup_printf (_("Plugin chain %s"),
+  title = g_strdup_printf (_("Plugin chain: %s"),
 			   gettext (chain_title[chain->type]));
   chain->dbox = gtk_dialog_new_with_buttons
 		  (title, NULL, 0,
