@@ -35,6 +35,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 #include "mediamarks.h"
 #include "preferences.h"
@@ -286,12 +289,50 @@ static void enum_cb (GtkWidget* widget, gpointer data)
   }
 }
 
+static void file_cb (GtkWidget *widget, int response, gpointer data)
+{
+  xine_cfg_entry_t entry;
+  gchar *key = (gchar *) data;
+
+  if (update_lock || response != GTK_RESPONSE_ACCEPT)
+    return;
+
+  logprintf ("preferences: file cb for key %s\n", key);
+
+  if (!xine_config_lookup_entry (xine, key, &entry))
+    return;
+
+  key = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (widget));
+  if (key && strcmp (entry.str_value, key))
+  {
+    logprintf ("preferences: updating entry\n");
+    entry.str_value = key;
+    xine_config_update_entry (xine, &entry);
+  }
+  free (key);
+}
+
+static void file_activate_cb (GtkWidget *widget, gpointer data)
+{
+  file_cb (widget, GTK_RESPONSE_ACCEPT, data);
+}
+
+static void file_map_cb (GtkWidget *fcb, gpointer key)
+{
+  if (!g_object_get_data (fcb, "gxine"))
+  {
+    g_signal_connect (fcb, "current-folder-changed",
+		      G_CALLBACK(file_activate_cb), key);
+    g_object_set_data (fcb, "gxine", fcb);
+  }
+}
 
 static GtkWidget *create_item_editable (const xine_cfg_entry_t *entry)
 {
   GtkWidget *widget;
 
-  switch (entry->type) {
+  switch (entry->type)
+  {
   case XINE_CONFIG_TYPE_ENUM:
     {
       int i;
@@ -310,13 +351,44 @@ static GtkWidget *create_item_editable (const xine_cfg_entry_t *entry)
     break;
 
   case XINE_CONFIG_TYPE_STRING:
-    widget = gtk_entry_new();
-    g_object_set_data (G_OBJECT(widget), "cfg", (gpointer)entry->key);
-    g_object_connect (G_OBJECT(widget),
+    if (xine_check_version (1, 1, 4) && entry->num_value)
+    {
+      widget = gtk_file_chooser_dialog_new (entry->key, GTK_WINDOW (prefs_dialog),
+					    entry->num_value == 3 /* XINE_CONFIG_STRING_IS_DIRECTORY_NAME */
+					      ? GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER
+					      : GTK_FILE_CHOOSER_ACTION_OPEN,
+					    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					    GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+					    NULL);
+      gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (widget), TRUE);
+      struct stat st;
+      if (stat (entry->str_value, &st))
+      {
+	g_printerr (_("warning: configuration item %s points to a non-existent location %s\n"),
+		  entry->key, entry->str_value);
+	xine_log (xine, 0, _("warning: configuration item %s points to a non-existent location %s\n"),
+		  entry->key, entry->str_value);
+      }
+      else
+	gtk_file_chooser_set_filename ((GtkFileChooser *) widget, entry->str_value);
+      g_object_connect (G_OBJECT(widget),
+	"signal::response", G_CALLBACK(file_cb), (gchar *)entry->key,
+	"signal::file-activated", G_CALLBACK(file_activate_cb), (gchar *)entry->key,
+	NULL);
+      widget = gtk_file_chooser_button_new_with_dialog (widget);
+      g_signal_connect (G_OBJECT(widget), "map", G_CALLBACK(file_map_cb), (gchar *)entry->key);
+    }
+    else
+    {
+      widget = gtk_entry_new();
+      g_object_set_data (G_OBJECT(widget), "cfg", (gpointer)entry->key);
+      g_object_connect (G_OBJECT(widget),
 	"signal::key-press-event", G_CALLBACK(entry_keypress_cb), (gchar *)entry->key,
 	"signal::focus-out-event", G_CALLBACK(entry_cb), (gchar *)entry->key,
 	NULL);
-    gtk_entry_set_text (GTK_ENTRY(widget), entry->str_value);
+      gtk_entry_set_text (GTK_ENTRY(widget), entry->str_value);
+    }
+
     break;
 
   case XINE_CONFIG_TYPE_RANGE: /* slider */
@@ -370,7 +442,7 @@ static GtkWidget *create_item_label (const xine_cfg_entry_t *entry)
   char *labeltext;
   const char *labelkey = strrchr (entry->key, '.') + 1;
 
-  if (entry->callback_data)
+  if (entry->callback)
     labeltext = g_markup_printf_escaped ("<b>%s</b>\n%s",
 					 labelkey, entry->description);
   else
@@ -495,6 +567,29 @@ static gboolean unmix_mixed_page (GNode *node, gboolean *done)
   return FALSE;
 }
 
+static gboolean focus_item_cb (GtkWidget *widget, GtkDirectionType *dir,
+			       GNode *node)
+{
+  const pref_t *pref = node->data;
+  const pref_t *parent = node->parent ? node->parent->data : NULL;
+  GtkWidget *viewport = parent->page.table->parent;
+  GtkAdjustment *adj = gtk_viewport_get_vadjustment ((GtkViewport *)viewport);
+
+  int wt = pref->item.editable->allocation.y;
+  int wb = wt + pref->item.editable->allocation.height;
+  int lt = pref->item.label->allocation.y;
+  int lb = lt + pref->item.label->allocation.height;
+  int top = wt < lt ? wt : lt;
+  int bottom = wb > lb ? wb : lb;
+
+  if (top < adj->value)
+    gtk_adjustment_set_value (adj, top);
+  else if (bottom + 4 > adj->value + viewport->allocation.height)
+    gtk_adjustment_set_value (adj, bottom - viewport->allocation.height + 4);
+  
+  return FALSE;
+}
+
 static gboolean put_content (GNode *node, gpointer data)
 {
   pref_t *pref = node->data;
@@ -545,6 +640,8 @@ static gboolean put_content (GNode *node, gpointer data)
 		      GTK_EXPAND | GTK_FILL, GTK_SHRINK, 5, 5);
     gtk_table_attach (table, pref->item.separator, 0, 4, r + 1, r + 2,
 		      GTK_EXPAND | GTK_FILL, GTK_SHRINK | GTK_FILL, 5, 5);
+    g_signal_connect (pref->item.editable, "focus",
+		      focus_item_cb, node);
   }
 
   return FALSE;
@@ -561,7 +658,8 @@ static GtkWidget *make_prefs_window (void)
   tooltips = gtk_tooltips_new ();
   prefs = create_page ("", 0); /* root page */
 
-  do {
+  do
+  {
     const char *point;
     GNode *section;
 
@@ -612,7 +710,7 @@ static void response_cb (GtkDialog *dbox, int response, gpointer data)
   switch (response)
   {
   case GTK_RESPONSE_ACCEPT:
-    fname = g_strconcat(g_get_home_dir(), "/.gxine/config", NULL);
+    fname = get_config_filename (FILE_CONFIG);
     xine_config_save (xine, fname);
     g_free (fname);
     break;
@@ -631,23 +729,8 @@ static JSBool js_preferences_show (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-void preferences_show (void)
+static gpointer preferences_init_dbox (gpointer unused)
 {
-  if (is_visible) {
-    is_visible = FALSE;
-    gtk_widget_hide (prefs_dialog);
-  } else {
-    is_visible = TRUE;
-    window_show (prefs_dialog, NULL);
-  }
-}
-
-
-void preferences_init (void)
-{
-  xine_cfg_entry_t entry;
-  GtkWidget *w;
-
   prefs_dialog = gtk_dialog_new_with_buttons (_("Preferences"), NULL, 0,
 				GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 				GTK_STOCK_CLOSE, GTK_RESPONSE_DELETE_EVENT,
@@ -658,27 +741,49 @@ void preferences_init (void)
 		    G_CALLBACK(response_cb), NULL);
 
   /* Make new tabbed box (notebook) */
+  GtkWidget *prefs_notebook = make_prefs_window ();
   gtk_box_pack_start_defaults (GTK_BOX(GTK_DIALOG(prefs_dialog)->vbox),
-			       make_prefs_window ());
+			       prefs_notebook);
 
-  w = gtk_label_new (_("Items marked “<b>• <i>like this</i></b>” require "
-		       "gxine to be restarted to take effect.")),
+  GtkWidget *w = gtk_label_new (_("Items marked “<b>• <i>like this</i></b>” require "
+				  "gxine to be restarted to take effect."));
   gtk_label_set_use_markup (GTK_LABEL(w), TRUE);
   gtk_box_pack_end (GTK_BOX(GTK_DIALOG(prefs_dialog)->vbox), w,
 		    FALSE, FALSE, 2);
-  gtk_widget_show_all (GTK_DIALOG(prefs_dialog)->vbox);
 
+  gtk_widget_show_all (prefs_notebook);
+
+  xine_cfg_entry_t entry;
   if (xine_config_lookup_entry (xine, "gui.experience_level", &entry))
   {
     static int value;
     value = entry.num_value;
-    g_idle_add ((GSourceFunc) select_show_pref_widgets, &value);
+    select_show_pref_widgets (&value);
   }
 
-  is_visible = 0;
+  return NULL;
+}
 
+void preferences_show (void)
+{
+  if (is_visible)
+  {
+    is_visible = FALSE;
+    gtk_widget_hide (prefs_dialog);
+  }
+  else
+  {
+    is_visible = TRUE;
+    static GOnce once = G_ONCE_INIT;
+    g_once (&once, preferences_init_dbox, NULL);
+    window_show (prefs_dialog, NULL);
+  }
+}
+
+void preferences_init (void)
+{
+  is_visible = FALSE;
   /* script engine functions */
-
   se_defun (gse, NULL, "preferences_show", js_preferences_show, 0, 0,
 	    SE_GROUP_DIALOGUE, NULL, NULL);
 }
@@ -688,13 +793,17 @@ void preferences_update_entry (const xine_cfg_entry_t *entry)
   gpointer widget = g_datalist_get_data (&prefs_map, entry->key);
 
   if (widget)
-    switch (entry->type) {
+    switch (entry->type)
+    {
     case XINE_CONFIG_TYPE_ENUM:
       gtk_combo_box_set_active (widget, entry->num_value);
       break;
 
     case XINE_CONFIG_TYPE_STRING:
-      gtk_entry_set_text (widget, entry->str_value);
+      if (xine_check_version (1, 1, 4) && entry->num_value)
+	gtk_file_chooser_set_filename (widget, entry->str_value);
+      else
+	gtk_entry_set_text (widget, entry->str_value);
       break;
 
     case XINE_CONFIG_TYPE_RANGE: /* slider */
