@@ -1,4 +1,4 @@
-/* 
+/*
  * Copyright (C) 2003-2006 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
@@ -250,11 +250,11 @@ se_js_string_val (JSContext *cx, const char *str)
   return STRING_TO_JSVAL (jstr);
 }
 
-/* 
+/*
  * methods
  */
 
-static JSBool controls_exit (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool controls_exit (JSContext *cx, JSObject *obj, uintN argc,
 			     jsval *argv, jsval *rval)
 {
   gchar *fname;
@@ -269,17 +269,21 @@ static JSBool controls_exit (JSContext *cx, JSObject *obj, uintN argc,
     exit (2);
   }
 
-  mm_save (); 
+  mm_save ();
   playlist_save (NULL);
   save_key_bindings ();
   save_startup_script ();
 
   logprintf ("script_engine: saving config...\n");
 
-  fname = g_strconcat(g_get_home_dir(), "/.gxine/config", NULL);
+  fname = get_config_filename (FILE_CONFIG);
   xine_config_save (xine, fname);
   g_free (fname);
 
+  fname = get_config_filename (FILE_ACCELS);
+  gtk_accel_map_save (fname);
+  g_free (fname);
+
   { /* lie about remembering the volume... */
     xine_cfg_entry_t entry;
     if (xine_config_lookup_entry (xine, "audio.volume.remember_volume", &entry))
@@ -472,7 +476,7 @@ static char *show_help_int (se_t *se, se_o_t *obj, se_group_t selector)
   return help;
 }
 
-static JSBool show_help (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool show_help (JSContext *cx, JSObject *obj, uintN argc,
 			  jsval *argv, jsval *rval)
 {
   static char *const group_id[] = {
@@ -569,7 +573,7 @@ static int get_prop_int (se_t *se, se_o_t *o, se_prop_t *p)
 static double get_prop_double (se_t *se, se_o_t *o, se_prop_t *p)
 {
   double n;
-  
+
   if (p->reader)
   {
     se_prop_read_t v;
@@ -657,7 +661,7 @@ static jsval get_prop_jsval (se_t *se, se_o_t *o, se_prop_t *p)
       JSBool b = get_prop_bool (se, o, p);
       return BOOLEAN_TO_JSVAL (b);
     }
- 
+
   case SE_TYPE_STRING:
     {
       const char *s = get_prop_string (se, o, p);
@@ -668,7 +672,7 @@ static jsval get_prop_jsval (se_t *se, se_o_t *o, se_prop_t *p)
     g_printerr (_("script_engine: error: property %s has unknown type %d\n"),
 	     p->id, p->se_type);
   }
- 
+
   return BOOLEAN_TO_JSVAL (0);
 }
 
@@ -701,7 +705,7 @@ static void set_prop_jsval (se_t *se, se_o_t *o, se_prop_t *p, const jsval *v)
       se->print_cb (se->print_cb_data, _("\n%s.%s: value must be boolean\n"),
 		    o->id, p->id);
     return;
- 
+
   case SE_TYPE_STRING:
     {
       JSString *str = JSVAL_TO_STRING(*v);
@@ -715,8 +719,8 @@ static void set_prop_jsval (se_t *se, se_o_t *o, se_prop_t *p, const jsval *v)
 	     p->id, p->se_type);
   }
 }
- 
-  
+
+
 /*
  * function to create and maintain js objects
  */
@@ -728,7 +732,8 @@ generic_JSGetProperty (JSContext *cx, JSObject *obj, jsval id, jsval *vp)
   se_log_fncall ("generic get property");
   se_o_t *o = JS_GetPrivate (cx, obj);
 
-  if (JSVAL_IS_STRING (id)) {
+  if (JSVAL_IS_STRING (id))
+  {
     JSString *str = JS_ValueToString (cx, id);
     char     *prop = str ? JS_GetStringBytes (str) : "";
     GList    *n;
@@ -737,28 +742,28 @@ generic_JSGetProperty (JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 	       prop, o->id);
 
     /* look through properties */
- 
+
     foreach_glist (n, o->properties)
     {
       se_prop_t *p = (se_prop_t *) n->data;
-      if (!strcasecmp (p->id, prop)) {
+      if (!strcasecmp (p->id, prop))
+      {
         *vp = get_prop_jsval (se, o, p);
         return JS_TRUE;
       }
     }
-     
+
     /* look through objects for default value */
- 
+
     foreach_glist (n, o->children)
     {
       se_o_t *p = (se_o_t *) n->data;
       if (!strcasecmp (p->id, prop))
       {
-        static const jschar name[] = { '.', 0 };
-	static const struct JSString
-	  __attribute__ ((aligned (1 << JSVAL_TAGBITS))) /* ugh */
-	  prop = { 1, name };
-	if (!generic_JSGetProperty (cx, p->obj, STRING_TO_JSVAL (&prop), vp))
+	static jsval prop = 0;
+	if (!prop)
+	  prop = se_js_string_val (cx, ".");
+	if (!generic_JSGetProperty (cx, p->obj, prop, vp))
 	  *vp = OBJECT_TO_JSVAL (p->obj);
 	return JS_TRUE;
       }
@@ -766,12 +771,12 @@ generic_JSGetProperty (JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 
     if (*prop && o->parent)
       return generic_JSGetProperty (cx, o->parent->obj, id, vp);
- 
+
     return JS_TRUE;
   }
- 
+
   *vp = JSVAL_VOID;
- 
+
   return JS_FALSE;
 }
 
@@ -780,12 +785,13 @@ generic_JSSetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 {
   se_t   *se = (se_t *) JS_GetContextPrivate(cx);
   se_o_t *o;
- 
+
   se_log_fncall ("generic set property");
- 
+
   o = JS_GetPrivate (cx, obj);
- 
-  if (JSVAL_IS_STRING (id)) {
+
+  if (JSVAL_IS_STRING (id))
+  {
     JSString *str = JS_ValueToString (cx, id);
     char     *prop = JS_GetStringBytes (str);
     GList    *n;
@@ -794,11 +800,12 @@ generic_JSSetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 	       prop, o->id);
 
     /* look through properties */
- 
+
     foreach_glist (n, o->properties)
     {
       se_prop_t *p = (se_prop_t *) n->data;
-      if (!strcasecmp (p->id, prop)) {
+      if (!strcasecmp (p->id, prop))
+      {
 	if (!p->constant)
 	  set_prop_jsval (se, o, p, vp);
         return JS_TRUE;
@@ -806,7 +813,7 @@ generic_JSSetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
     }
 
     /* look through objects for default value */
- 
+
     foreach_glist (n, o->children)
     {
       se_o_t *p = (se_o_t *) n->data;
@@ -823,22 +830,23 @@ generic_JSSetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
 
     return JS_TRUE;
   }
- 
+
   *vp = JSVAL_VOID;
-  
+
   return JS_FALSE;
 }
 
-static void generic_JSDestructor (JSContext *cx, JSObject *obj) {
+static void generic_JSDestructor (JSContext *cx, JSObject *obj)
+{
   se_log_fncall ("generic destructor");
 }
 
-static JSClass generic_JSClass = 
+static JSClass generic_JSClass =
   {
     "view", JSCLASS_HAS_PRIVATE,
     JS_PropertyStub, JS_PropertyStub,
     generic_JSGetProperty, generic_JSSetProperty,
-    JS_EnumerateStub, JS_ResolveStub, 
+    JS_EnumerateStub, JS_ResolveStub,
     JS_ConvertStub, generic_JSDestructor
   };
 
@@ -940,7 +948,7 @@ static void se_prop_listeners (se_t *se, se_o_t *o, se_prop_t *p, se_prop_read_t
     l->cb (l->user_data, se, o, p, r);
   }
 }
- 
+
 void se_prop_set (se_t *se, se_o_t *o, const char *id, const char *value)
 {
   se_prop_t *p;
@@ -1045,16 +1053,14 @@ void se_prop_set_bool (se_t *se, se_o_t *o, const char *id, int value)
     se_prop_create_bool (se, o, id, value, FALSE);
 }
 
-char *se_prop_get (se_t *se, se_o_t *o, const char *id) {
- 
-  se_prop_t *p;
- 
+char *se_prop_get (se_t *se, se_o_t *o, const char *id)
+{
   if (!o)
     o = se->g;
- 
-  p = se_prop_find (se, o, id);
- 
-  if (!p) {
+
+  se_prop_t *p = se_prop_find (se, o, id);
+  if (!p)
+  {
     g_printerr (_("script_engine: se_prop_get%s error: property %s doesn't exist\n"),
 	     "", se_id (se, o, id));
     return NULL;
@@ -1087,13 +1093,12 @@ char *se_prop_get (se_t *se, se_o_t *o, const char *id) {
 
 int se_prop_get_int (se_t *se, se_o_t *o, const char *id)
 {
-  se_prop_t *p;
-
   if (!o)
     o = se->g;
 
-  p = se_prop_find (se, o, id);
-  if (!p) {
+  se_prop_t *p = se_prop_find (se, o, id);
+  if (!p)
+  {
     g_printerr (_("script_engine: se_prop_get%s error: property %s doesn't exist\n"),
 	     "_int", se_id (se, o, id));
     return 0;
@@ -1122,13 +1127,12 @@ int se_prop_get_int (se_t *se, se_o_t *o, const char *id)
 
 double se_prop_get_double (se_t *se, se_o_t *o, const char *id)
 {
-  se_prop_t *p;
-
   if (!o)
     o = se->g;
 
-  p = se_prop_find (se, o, id);
-  if (!p) {
+  se_prop_t *p = se_prop_find (se, o, id);
+  if (!p)
+  {
     g_printerr (_("script_engine: se_prop_get%s error: property %s doesn't exist\n"),
 	     "_double", se_id (se, o, id));
     return 0;
@@ -1136,7 +1140,7 @@ double se_prop_get_double (se_t *se, se_o_t *o, const char *id)
 
   logprintf ("script_engine: getting value of %s=='%s'\n",
 	     se_id (se, o, id), p->value);
- 
+
   switch (p->se_type)
   {
   case SE_TYPE_INT:
@@ -1157,13 +1161,12 @@ double se_prop_get_double (se_t *se, se_o_t *o, const char *id)
 
 JSBool se_prop_get_bool (se_t *se, se_o_t *o, const char *id)
 {
-  se_prop_t *p;
-
   if (!o)
     o = se->g;
 
-  p = se_prop_find (se, o, id);
-  if (!p) {
+  se_prop_t *p = se_prop_find (se, o, id);
+  if (!p)
+  {
     g_printerr (_("script_engine: se_prop_get%s error: property %s doesn't exist\n"),
 	     "_bool", se_id (se, o, id));
     return JS_FALSE;
@@ -1171,7 +1174,7 @@ JSBool se_prop_get_bool (se_t *se, se_o_t *o, const char *id)
 
   logprintf ("script_engine: getting value of %s=='%s'\n",
 	     se_id (se, o, id), p->value);
- 
+
   switch (p->se_type)
   {
   case SE_TYPE_INT:
@@ -1195,15 +1198,14 @@ void se_prop_create (se_t *se, se_o_t *o,
 		     const char *xine_id, int xine_param,
 		     se_prop_type_t se_type, gboolean constant)
 {
-  se_prop_t *p;
- 
   if (!o)
     o = se->g;
 
   logprintf ("script_engine: setting %s to '%s'\n", se_id (se, o, id), value);
 
-  p = se_prop_find (se, o, id);
-  if (!p) {
+  se_prop_t *p = se_prop_find (se, o, id);
+  if (!p)
+  {
     logprintf ("script_engine: creating new property %s\n", se_id (se, o, id));
     p = malloc (sizeof (se_prop_t));
     p->id = strdup (id);
@@ -1227,7 +1229,7 @@ void se_prop_create_int (se_t *se, se_o_t *o, const char *id, int value,
   se_prop_create (se, o, id, str, NULL, 0, SE_TYPE_INT, constant);
 }
 
-void se_prop_create_double (se_t *se, se_o_t *o, const char *id, double value, 
+void se_prop_create_double (se_t *se, se_o_t *o, const char *id, double value,
 			    gboolean constant)
 {
   char str[24];
@@ -1235,7 +1237,7 @@ void se_prop_create_double (se_t *se, se_o_t *o, const char *id, double value,
   se_prop_create (se, o, id, str, NULL, 0, SE_TYPE_DOUBLE, constant);
 }
 
-void se_prop_create_bool (se_t *se, se_o_t *o, const char *id, int value, 
+void se_prop_create_bool (se_t *se, se_o_t *o, const char *id, int value,
 			  gboolean constant)
 {
   char str[24];
@@ -1314,7 +1316,7 @@ void se_prop_add_listener (se_t *se, se_o_t *o, const char *id,
 			   se_prop_cb_t prop_cb, void *user_data)
 {
   se_prop_t *p;
- 
+
   if (!o)
     o = se->g;
 
@@ -1337,7 +1339,7 @@ void se_prop_set_reader_func (se_t *se, se_o_t *o, const char *id,
 			      se_prop_reader_t reader)
 {
   se_prop_t *p;
- 
+
   if (!o)
     o = se->g;
 
@@ -1348,7 +1350,7 @@ void se_prop_set_reader_func (se_t *se, se_o_t *o, const char *id,
     g_printerr (_("script_engine: error in %s: property %s not found\n"),
 	     __func__, se_id (se, o, id));
   else
-    p->reader = reader;  
+    p->reader = reader;
 }
 
 static void error_reporter (JSContext *cx, const char *message,
@@ -1370,10 +1372,13 @@ se_f_t *se_defun (se_t *se, se_o_t *o /* may be NULL */,
   f = malloc (sizeof(se_f_t));
   f->id = strdup (name);
 
-  if (o) {
+  if (o)
+  {
     f->fun = JS_DefineFunction (se->cx, o->obj, name, fun, nargs, attrs);
     o->functions = g_list_append (o->functions, f);
-  } else {
+  }
+  else
+  {
     f->fun = JS_DefineFunction (se->cx, se->global, name, fun, nargs, attrs);
     se->g->functions = g_list_append (se->g->functions, f);
   }
@@ -1541,10 +1546,10 @@ se_t *se_new (void)
     se_create_console ();
 
   /*
-   * create a js engine instance 
+   * create a js engine instance
    */
 
-  if (!rt) 
+  if (!rt)
     rt = JS_NewRuntime(0x100000);
 
   se = malloc (sizeof (se_t));
@@ -1554,7 +1559,11 @@ se_t *se_new (void)
 
   pthread_mutexattr_t attr;
   pthread_mutexattr_init (&attr);
+#ifdef PTHREAD_MUTEX_RECURSIVE_NP
   pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_RECURSIVE_NP);
+#else /* *BSD does not have the _NP in the name */
+  pthread_mutexattr_settype (&attr, PTHREAD_MUTEX_RECURSIVE);
+#endif
   pthread_mutex_init (&se->lock, &attr);
   pthread_mutexattr_destroy (&attr);
 
@@ -1565,7 +1574,7 @@ se_t *se_new (void)
   /*
    * create fake global se_o
    */
- 
+
   se->g = malloc (sizeof(se_o_t));
   se->g->obj        = se->global;
   se->g->parent     = NULL;
