@@ -14,9 +14,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: server.h,v 1.2 2003/03/20 21:59:58 guenter Exp $
+ * $Id: server.h,v 1.3 2006-02-21 19:27:47 dsalt Exp $
  *
  * gxine remote control via socket
  */
@@ -26,7 +26,7 @@
 
 /* functions to attempt to connect to already running gxine */
 int  server_client_connect (void);
-void server_client_send (char *cmd);
+int  server_client_send (const char *cmd);
 void server_client_stop (void);
 
 /* if no gxine is running, create our own gxine socket: */
