@@ -37,28 +37,23 @@ static GtkWidget    *dlg;
 static int           is_visible;
 static GtkListStore *meta_store;
 
-static void add_meta_entry (const gchar *l, int info) {
-
+static void add_meta_entry (const gchar *l, int info)
+{
   GtkTreeIter  iter;
-  const char *str;
-
-  str = xine_get_meta_info (stream, info);
+  const char *str = xine_get_meta_info (stream, info);
   if (!str)
     return;
 
   gtk_list_store_append (meta_store, &iter);
-
-  gtk_list_store_set (meta_store, &iter, 0, l, 1, 
+  gtk_list_store_set (meta_store, &iter, 0, l, 1,
 		      g_utf8_normalize (str, -1, G_NORMALIZE_DEFAULT), -1);
 }
 
-static void add_string_entry (const gchar *l, char *info) {
-
+static void add_string_entry (const gchar *l, char *info)
+{
   GtkTreeIter  iter;
-
   gtk_list_store_append (meta_store, &iter);
-
-  gtk_list_store_set (meta_store, &iter, 0, l, 1, 
+  gtk_list_store_set (meta_store, &iter, 0, l, 1,
 		      g_utf8_normalize (info, -1, G_NORMALIZE_DEFAULT), -1);
 }
 
@@ -106,7 +101,8 @@ static void update_metadata (void)
 
   add_meta_entry (_("Audio codec:"), XINE_META_INFO_AUDIOCODEC);
   count = xine_get_stream_info (stream, XINE_STREAM_INFO_AUDIO_CHANNELS);
-  if (count) {
+  if (count)
+  {
     int bits = xine_get_stream_info (stream, XINE_STREAM_INFO_AUDIO_BITS);
     int rate = xine_get_stream_info (stream, XINE_STREAM_INFO_AUDIO_SAMPLERATE);
     int bps  = xine_get_stream_info (stream, XINE_STREAM_INFO_AUDIO_BITRATE);
@@ -186,7 +182,7 @@ static void response_cb (GtkDialog *dbox, int response, gpointer data)
   }
 }
 
-static JSBool js_stream_info_show (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_stream_info_show (JSContext *cx, JSObject *obj, uintN argc,
 				   jsval *argv, jsval *rval)
 {
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
@@ -195,8 +191,8 @@ static JSBool js_stream_info_show (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-void stream_info_init (void) {
-
+void stream_info_init (void)
+{
   GtkTreeView	       *tree_view;
   GtkWidget	       *scrolled_window;
   GtkCellRenderer      *cell;
@@ -215,7 +211,7 @@ void stream_info_init (void) {
   /* add list to hold meta info */
 
   meta_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
-  
+
   tree_view = GTK_TREE_VIEW (gtk_tree_view_new_with_model (GTK_TREE_MODEL (meta_store)));
   gtk_tree_view_set_rules_hint (tree_view, TRUE);
 
@@ -232,7 +228,7 @@ void stream_info_init (void) {
 
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
-				  GTK_POLICY_AUTOMATIC, 
+				  GTK_POLICY_AUTOMATIC,
 				  GTK_POLICY_AUTOMATIC);
   gtk_container_add (GTK_CONTAINER (scrolled_window), GTK_WIDGET (tree_view));
 
@@ -244,4 +240,3 @@ void stream_info_init (void) {
   se_defun (gse, NULL, "stream_info_show", js_stream_info_show, 0, 0,
 	    SE_GROUP_DIALOGUE, N_("[bool]"), NULL);
 }
-
