@@ -1,8 +1,8 @@
 /*
  * Copyright (C) 2004-2006 the xine-project
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
@@ -356,6 +356,41 @@ GtkWidget *ui_button_new_stock_mnemonic (const char *stock,
   return button;
 }
 
+static GtkWidget *ui_container_new_icon_name (GtkWidget *box, const char *name)
+{
+  GtkWidget *img = gtk_image_new_from_icon_name (name, GTK_ICON_SIZE_BUTTON);
+  gtk_misc_set_padding (GTK_MISC(img), 0, 0);
+  if (box)
+    gtk_container_add (GTK_CONTAINER(box), img);
+  return box ? box : img;
+}
+
+GtkWidget *ui_button_new_icon_name (const char *name)
+{
+  return ui_container_new_icon_name (gtk_button_new (), name);
+}
+
+GtkWidget *ui_toggle_button_new_icon_name (const char *name)
+{
+  return ui_container_new_icon_name (gtk_toggle_button_new (), name);
+}
+
+GtkWidget *ui_button_new_icon_name_mnemonic (const char *name,
+					     const char *mnemonic)
+{
+  GtkWidget *button = gtk_button_new ();
+  GtkWidget *align = gtk_alignment_new (0.5, 0.5, 0, 0);
+  GtkWidget *hbox = gtk_hbox_new (FALSE, 2);
+  /* hbox in alignment in button => hbox is not expanded to fill the button */
+  gtk_container_add (GTK_CONTAINER(align), hbox);
+  gtk_container_add (GTK_CONTAINER(button), align);
+  gtk_box_pack_start (GTK_BOX(hbox), ui_container_new_icon_name (NULL, name),
+		      FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX(hbox), gtk_label_new_with_mnemonic (mnemonic),
+		      FALSE, FALSE, 0);
+  return button;
+}
+
 /* Menu/toolbar basics */
 
 GtkUIManager *ui_create_manager (const char *label, GtkWidget *window)
@@ -418,10 +453,10 @@ static void set_fs_toolbar_top (int v)
   else
     gtk_action_activate (action_items.fs_toolbar_pos->data); /* bottom */
 }
-            
+
 #ifdef WITH_DEPRECATED
 
-static JSBool js_fs_toolbar_show (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_fs_toolbar_show (JSContext *cx, JSObject *obj, uintN argc,
 				  jsval *argv, jsval *rval)
 {
   JSBool show;
@@ -442,7 +477,7 @@ static JSBool js_fs_toolbar_show (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-static JSBool js_fs_toolbar_position (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_fs_toolbar_position (JSContext *cx, JSObject *obj, uintN argc,
 				      jsval *argv, jsval *rval)
 {
   JSBool top;
@@ -550,7 +585,7 @@ static JSBool ui_prop_toggle_bool (JSContext *cx, JSObject *obj,
   return JS_TRUE;
 }
 
-void ui_create_properties (const ui_property_t *prop, se_o_t *parent, 
+void ui_create_properties (const ui_property_t *prop, se_o_t *parent,
 			   se_prop_type_t type)
 {
   int i;
@@ -639,7 +674,7 @@ void ui_preferences_register (xine_t *this)
     (this, "gui.experience_level", 0, experience_labels,
      _("Display of configuration settings"),
      _("Controls whether more advanced configuration settings are shown."),
-     0, NULL, CONFIG_DATA_NONE);
+     0, gtk_true, NULL);
 
   /* Register a few audio/video config items */
   for (i = 0; i < G_N_ELEMENTS (ranges); ++i)
@@ -702,7 +737,7 @@ void ui_preferences_register (xine_t *this)
   xine_config_register_bool
     (this, "gui.post_plugins.always_show_logo", 1,
      _("In windowed mode, whether the logo is shown after playing an audio-only stream if no audio visualisation was active."), NULL,
-     0, NULL, NULL);
+     0, gtk_true, NULL);
 
   xine_config_register_enum
     (this, "gui.fullscreen_toolbar", 1, tbar_pos_labels,
@@ -714,6 +749,11 @@ void ui_preferences_register (xine_t *this)
      _("In windowed mode, whether the toolbar is in a separate window"), NULL,
      0, NULL, NULL);
 
+  xine_config_register_bool
+    (this, "gui.windowedmode_unblank", 0,
+     _("In windowed mode, prevent blanking when playing video"), NULL,
+     0, NULL, NULL);
+
   xine_config_register_bool
     (this, "gui.show_splash", 1,
      _("Display splash screen"),
@@ -739,8 +779,8 @@ static GtkIconSource *make_icon_source (const char *name, GtkStateType state,
 {
   GtkIconSource *isrc = gtk_icon_source_new ();
   char *filename = *name == '/'
-		 ? g_strconcat (icondir, name, NULL)
-		 : g_strconcat (pixmapdir, "/", name, NULL);
+		   ? g_build_filename (icondir, name, NULL)
+		   : g_build_filename (pixmapdir, name, NULL);
   gtk_icon_source_set_filename (isrc, filename);
   free (filename);
   gtk_icon_source_set_direction_wildcarded (isrc, TRUE);
@@ -908,7 +948,7 @@ void ui_init (void)
 
   toolbars = se_create_object (gse, NULL, "toolbar", NULL,
 			       SE_GROUP_PROPERTIES, NULL);
-  
+
   ui_create_properties (tb_props, toolbars, SE_TYPE_BOOL);
   ui_create_properties (tb_fs_props, se_find_object (gse, toolbars, "fs"),
 			SE_TYPE_BOOL);
