@@ -1,8 +1,8 @@
 /*
  * Copyright (C) 2001-2006 the xine-project
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
@@ -153,14 +153,15 @@ static gboolean do_health_check (GtkWidget *dlg, GtkWidget *vbox)
     if (results->status == XINE_HEALTH_CHECK_NO_SUCH_CHECK)
       break;
 
-    gtk_table_resize ( GTK_TABLE (table), 3, check+1);
+    gtk_table_resize (GTK_TABLE (table), 3, check+1);
 
     l = gtk_label_new (results->title);
     gtk_table_attach (GTK_TABLE (table), l, 0, 1, check, check+1, GTK_FILL,
 		      GTK_SHRINK, 2, 5);
     gtk_misc_set_alignment (GTK_MISC (l), 0, 0.5);
 
-    switch (results->status) {
+    switch (results->status)
+    {
     case XINE_HEALTH_CHECK_UNSUPPORTED:
     case XINE_HEALTH_CHECK_OK:
       l = gtk_image_new_from_stock (GTK_STOCK_YES, GTK_ICON_SIZE_BUTTON);
@@ -197,11 +198,14 @@ static GtkWidget *cb_mailcap;
 static void integrate_cb (GtkButton *button, gpointer data)
 {
   gboolean state = !!data;
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_mailcap), state);
+  if (GTK_WIDGET_SENSITIVE (cb_mailcap))
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_mailcap), state);
   if (GTK_WIDGET_SENSITIVE (cb_moz))
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_moz), state);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_gnome), state);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_kde), state);
+  if (GTK_WIDGET_SENSITIVE (cb_gnome))
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_gnome), state);
+  if (GTK_WIDGET_SENSITIVE (cb_kde))
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_kde), state);
 }
 
 static gboolean ask_integration_wizard (GtkWidget *w, GtkWidget *vbox)
@@ -209,7 +213,7 @@ static gboolean ask_integration_wizard (GtkWidget *w, GtkWidget *vbox)
   struct stat st;
   gchar *sname;
   GtkWidget *b, *hbox;
-  gboolean have_plugin;
+  gboolean enable;
 
   add_heading (vbox, _("Registration"));
   add_label (vbox, _("Register gxine with the following applications "
@@ -221,15 +225,17 @@ static gboolean ask_integration_wizard (GtkWidget *w, GtkWidget *vbox)
 
   cb_moz = gtk_check_button_new_with_mnemonic (_("_Mozilla & Mozilla Firefox (plugin)"));
   gtk_box_pack_start (GTK_BOX(vbox), cb_moz, FALSE, FALSE, 5);
-  sname = g_strconcat (plugindir, "/gxineplugin.so", NULL);
-  have_plugin = !stat (sname, &st);
+  sname = g_build_filename (plugindir, "gxineplugin.so", NULL);
+  enable = !stat (sname, &st);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_moz), enable);
+  gtk_widget_set_sensitive (cb_moz, enable);
   g_free (sname);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_moz), have_plugin);
-  gtk_widget_set_sensitive (cb_moz, have_plugin);
 
   cb_gnome = gtk_check_button_new_with_mnemonic (_("_GNOME, Nautilus"));
   gtk_box_pack_start (GTK_BOX(vbox), cb_gnome, FALSE, FALSE, 5);
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_gnome), TRUE);
+  enable = di_gnome_vfs_init ();
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cb_gnome), enable);
+  gtk_widget_set_sensitive (cb_gnome, enable);
 
   cb_kde = gtk_check_button_new_with_mnemonic (_("_KDE, Konqueror"));
   gtk_box_pack_start (GTK_BOX(vbox), cb_kde, FALSE, FALSE, 5);
@@ -254,18 +260,14 @@ static gboolean ask_integration_wizard (GtkWidget *w, GtkWidget *vbox)
 static void desktop_integration (void)
 {
   di_registration_flush ();
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_mailcap))) {
+  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_mailcap)))
     di_register_mailcap();
-  }
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_gnome))) {
+  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_gnome)))
     di_register_gnome();
-  }
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_kde))) {
+  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_kde)))
     di_register_kde();
-  }
-  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_moz))) {
+  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (cb_moz)))
     di_register_mozilla();
-  }
 }
 
 static gboolean report_integration_wizard (GtkWidget *w, GtkWidget *vbox)
@@ -286,7 +288,8 @@ static gboolean report_integration_wizard (GtkWidget *w, GtkWidget *vbox)
 #define RESPONSE_CLOSE 0
 #define RESPONSE_NEXT  1
 
-static gboolean close_cb (GtkWidget* widget, gpointer data) {
+static gboolean close_cb (GtkWidget* widget, gpointer data)
+{
   int *b = data;
   *b = RESPONSE_CLOSE;
   gtk_main_quit();
@@ -342,7 +345,8 @@ void run_wizards (gboolean requested)
       { NULL }
     };
 
-    if (xine_config_lookup_entry (xine, "misc.wizards_shown", &entry)) {
+    if (xine_config_lookup_entry (xine, "misc.wizards_shown", &entry))
+    {
       entry.num_value = WIZARDS_LEVEL;
       preferences_update_entry (&entry);
     }
@@ -376,10 +380,10 @@ void run_wizards (gboolean requested)
       gtk_label_set_use_markup (GTK_LABEL (w), TRUE);
       gtk_box_pack_start (GTK_BOX (b), w, FALSE, FALSE, 2);
     }
-    
+
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox),
 			gtk_hseparator_new (), FALSE, FALSE, 5);
-    
+
     window_show (dlg, NULL);
 
     state = -1;
@@ -417,7 +421,7 @@ void run_wizards (gboolean requested)
   }
 }
 
-static JSBool js_run_wizards (JSContext *cx, JSObject *obj, uintN argc, 
+static JSBool js_run_wizards (JSContext *cx, JSObject *obj, uintN argc,
 			      jsval *argv, jsval *rval)
 {
   /* se_t *se = (se_t *) JS_GetContextPrivate(cx); */
@@ -426,8 +430,8 @@ static JSBool js_run_wizards (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-void wizards_init (void) {
+void wizards_init (void)
+{
   se_defun (gse, NULL, "run_wizards", js_run_wizards, 0, 0,
 	    SE_GROUP_HIDDEN, NULL, NULL);
 }
-
